@@ -1,7 +1,8 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2019-2022 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2019-2025 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2020 Björn Höfling <bjoern.hoefling@bjoernhoefling.de>
-;;; Copyright © 2022 Maxim Cournoyer <maxim.cournoyer@gmail.com>
+;;; Copyright © 2022 Maxim Cournoyer <maxim@guixotic.coop>
+;;; Copyright © 2025 Florian Pelz <pelzflorian@pelzflorian.de>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -36,6 +37,7 @@
              (git)
              (gnu packages base)
              (gnu packages compression)
+             (gnu packages fonts)
              (gnu packages gawk)
              (gnu packages gettext)
              (gnu packages guile)
@@ -66,11 +68,11 @@
 
 (define %manual-languages
   ;; Available translations for the 'guix-manual' text domain.
-  '("de" "en" "es" "fr" "pt_BR" "ru" "zh_CN"))
+  '("de" "en" "es" "fr" "it" "pt_BR" "ru" "zh_CN"))
 
 (define %cookbook-languages
   ;; Available translations for the 'guix-cookbook' text domain.
-  '("de" "en" "fr" "sk"))
+  '("de" "en" "es" "fr" "it" "ko" "pt_BR" "ru" "sk" "sv" "ta" "zh_CN"))
 
 (define %languages
   ;; Available translations for the document being built.
@@ -78,6 +80,10 @@
       %cookbook-languages
       %manual-languages))
 
+(define %latest-guix-version
+  ;; Latest released version.
+  "1.5.0")
+
 (define (texinfo-manual-images source)
   "Return a directory containing all the images used by the user manual, taken
 from SOURCE, the root of the source tree."
@@ -129,11 +135,12 @@ as well as images, OS examples, and translations."
 
   (define examples
     (file-append* source "gnu/system/examples"))
-
   (define build
     (with-imported-modules '((guix build utils))
       #~(begin
           (use-modules (guix build utils)
+                       (ice-9 binary-ports)
+                       (ice-9 textual-ports)
                        (srfi srfi-19))
 
           (define (make-version-texi language)
@@ -155,6 +162,22 @@ as well as images, OS examples, and translations."
                             (date->string date "~B ~Y")
                             version version))))))
 
+          (define (replace-first-line file replacement)
+            ;; Modify first line of FILE to REPLACEMENT.
+            (with-atomic-file-replacement
+             file
+             (lambda (in out)
+               (get-line in)
+               (display replacement out)
+               (newline out)
+               (letrec ((loop (lambda ()
+                                (let ((bv (get-bytevector-n in 65536)))
+                                  (unless (eof-object? bv)
+                                    (begin
+                                      (put-bytevector out bv)
+                                      (loop)))))))
+                 (loop)))))
+
           (install-file #$(file-append documentation "/htmlxref.cnf")
                         #$output)
 
@@ -164,6 +187,15 @@ as well as images, OS examples, and translations."
                             (find-files #$(translated-texi-manuals source)
                                         "\\.texi$")))
 
+          ;; For CJK support in PDF, replace \input line.
+          (with-directory-excursion #$output
+            (replace-first-line
+             "guix.zh_CN.texi"
+             "\\input texinfo-zh")
+            (replace-first-line
+             "guix-cookbook.zh_CN.texi"
+             "\\input texinfo-zh"))
+
           ;; Create 'version.texi'.
           (for-each make-version-texi '#$languages)
 
@@ -184,11 +216,16 @@ as well as images, OS examples, and translations."
 (define %web-site-url
   ;; URL of the web site home page.
   (or (getenv "GUIX_WEB_SITE_URL")
-      "/software/guix/"))
+      "/"))
+
+(define %manual-css-url
+  ;; URL of the main CSS file.
+  (in-vicinity %web-site-url
+               "themes/initial/css/manual.css"))
 
 (define %makeinfo-html-options
   ;; Options passed to 'makeinfo --html'.
-  '("--css-ref=https://www.gnu.org/software/gnulib/manual.css"
+  `(,(string-append "--css-ref=" %manual-css-url)
     "-c" "EXTRA_HEAD=<meta name=\"viewport\" \
 content=\"width=device-width, initial-scale=1\" />"))
 
@@ -284,6 +321,9 @@ actual file name."
                      (loop rest))
                     ((('strong _ ...) _ ...)
                      #t)
+                    ((('span ('@ ('class "category")) ;raw Texinfo 6.8
+                             (? string-or-entity?) ...) rest ...)
+                     #t)
                     ((('span ('@ ('class "symbol-definition-category"))
                              (? string-or-entity?) ...) rest ...)
                      #t)
@@ -362,7 +402,7 @@ actual file name."
                                                             #:languages
                                                             languages))
                                   (syntax-css-url
-                                   "/static/base/css/code.css"))
+                                   "/themes/initial/css/code.css"))
   "Return a derivation called NAME that processes all the HTML files in INPUT
 to (1) add them a link to SYNTAX-CSS-URL, and (2) highlight the syntax of all
 its <pre class=\"lisp\"> blocks (as produced by 'makeinfo --html')."
@@ -507,10 +547,16 @@ its <pre class=\"lisp\"> blocks (as produced by 'makeinfo --html')."
 
                   ;; Replace the ugly <strong> used for @deffn etc., which
                   ;; translate to <dt>, with more stylable markup.
-                  (('dt (@ ('id id)) category ... ('strong thing))
+                  (('dt ('@ ('id id))             ;raw Texinfo 6.8
+                        ('span ('@ ('class "category")) category ...)
+                        ('span ('strong thing)
+                               anchor))
                    (highlight-definition id category thing '()))
-                  (('dt (@ ('id id)) category ... ('strong thing)
-                        (? space?) ('em args ...))
+                  (('dt (@ ('id id))
+                        ('span ('@ ('class "category")) category ...)
+                        ('span ('strong thing)
+                               (? space?) ('em args ...)
+                               anchor))
                    (highlight-definition id category thing args))
 
                   ((tag ('@ attributes ...) body ...)
@@ -613,9 +659,10 @@ its <pre class=\"lisp\"> blocks (as produced by 'makeinfo --html')."
 
 (define* (stylized-html source input
                         #:key
+                        (latest-version %latest-guix-version)
                         (languages %languages)
                         (manual %manual)
-                        (manual-css-url "/static/base/css/manual.css"))
+                        (manual-css-url %manual-css-url))
   "Process all the HTML files in INPUT; add them MANUAL-CSS-URL as a <style>
 link, and add a menu to choose among LANGUAGES.  Use the Guix PO files found
 in SOURCE."
@@ -661,6 +708,14 @@ in SOURCE."
 	                 (href ,url))
                       ,label)))
 
+            (define menu-item-separator
+              ;; Thin horizontal line to separate drop-down menu items.
+              `(img (@ (class "hline")
+                       (src ,(in-vicinity
+                              #$%web-site-url
+                              "themes/initial/img/h-separator.png"))
+                       (alt ""))))
+
             (define* (navigation-bar menus #:key split-node?)
               ;; Return the navigation bar showing all of MENUS.
               `(header (@ (class "navbar"))
@@ -704,10 +759,46 @@ in SOURCE."
                   (list
                    (menu-item "⊕"
                               (if (string=? #$manual "guix-cookbook")
-                                  "https://translate.fedoraproject.org/projects/guix/documentation-cookbook/"
-                                  "https://translate.fedoraproject.org/projects/guix/documentation-manual/")))))
+                                  "https://translate.codeberg.org/projects/guix/documentation-cookbook/"
+                                  "https://translate.codeberg.org/projects/guix/documentation-manual/")))))
 
-            (define (stylized-html sxml file)
+            (define (version-menu-items language split-node?)
+              ;; Return the menu items to select the version of the manual of
+              ;; the type of medium (PDF, split-node, etc.).
+              (define language-extension
+                (if (string=? language "en")
+                    ""
+                    (string-append "." language)))
+
+              (define pdf-link
+                (string-append (if split-node? "../" "")
+                               #$manual language-extension ".pdf"))
+
+              (define version-links
+                (list (menu-item #$latest-version
+                                 (string-append
+                                  "/manual/" #$latest-version
+                                  "/" language
+                                  (if split-node? "/html_node" "")))
+                      (menu-item "development"
+                                 (string-append
+                                  "/manual/devel/" language
+                                  (if split-node? "/html_node" "")))
+                      menu-item-separator))
+
+              (append (if (string=? #$manual "guix")
+                          version-links
+                          '())
+                      (list (if split-node?
+                                (menu-item "single page"
+                                           (string-append "../" #$manual
+                                                          language-extension
+                                                          ".html"))
+                                (menu-item "multiple pages"
+                                           "html_node"))
+                            (menu-item "PDF" pdf-link))))
+
+            (define (stylized-html sxml file language)
               ;; Return SXML, which was read from FILE, with additional
               ;; styling.
               (define split-node?
@@ -730,9 +821,16 @@ in SOURCE."
                             ;; TODO: Add "Contribute" menu, to report
                             ;; errors, etc.
                             (list (menu-dropdown
+                                   #:label "Version"
+                                   #:items
+                                   (version-menu-items language
+                                                       split-node?))
+                                  (menu-dropdown
                                    #:label
                                    `(img (@ (alt "Language")
-                                            (src "/static/base/img/language-picker.svg")))
+                                            (src #$(in-vicinity
+                                                    %web-site-url
+                                                    "themes/initial/img/language-picker.svg"))))
                                    #:items
                                    (language-menu-items file)))
                             #:split-node? split-node?)
@@ -744,13 +842,13 @@ in SOURCE."
                   ((? string? str)
                    str))))
 
-            (define (process-html file)
+            (define (process-html file language)
               ;; Parse FILE and add links to translations.  Install the result
               ;; to #$output.
               (format (current-error-port) "processing ~a...~%" file)
               (let* ((shtml        (parameterize ((%strict-tokenizer? #t))
                                      (call-with-input-file file html->shtml)))
-                     (processed    (stylized-html shtml file))
+                     (processed    (stylized-html shtml file language))
                      (base         (string-drop file (string-length #$input)))
                      (target       (string-append #$output base)))
                 (mkdir-p (dirname target))
@@ -758,6 +856,15 @@ in SOURCE."
                   (lambda (port)
                     (write-shtml-as-html processed port)))))
 
+            (define (input-file-language file)
+              ;; Return the language code of FILE, an input file, as a string
+              ;; like "sv" or "zh-cn".
+              (match (string-tokenize (string-drop file
+                                                   (string-length #$input))
+                                      (char-set-complement
+                                       (char-set #\/)))
+                ((language _ ...) language)))
+
             ;; Install a UTF-8 locale so we can process UTF-8 files.
             (setenv "GUIX_LOCPATH"
                     #+(file-append glibc-utf8-locales "/lib/locale"))
@@ -768,7 +875,8 @@ in SOURCE."
             (n-par-for-each (parallel-job-count)
                             (lambda (file)
                               (if (string-suffix? ".html" file)
-                                  (process-html file)
+                                  (let ((language (input-file-language file)))
+                                    (process-html file language))
                                   ;; Copy FILE as is to #$output.
                                   (let* ((base   (string-drop file (string-length #$input)))
                                          (target (string-append #$output base)))
@@ -835,7 +943,8 @@ makeinfo OPTIONS."
                       (let* ((texi (language->texi-file-name language))
                              (opts `("--html"
                                      "-c" ,(string-append "TOP_NODE_UP_URL=/manual/"
-                                                         language)
+                                                          #$%latest-guix-version
+                                                          "/" language)
                                      #$@options
                                      ,texi)))
                         (format #t "building HTML manual for language '~a'...~%"
@@ -895,19 +1004,22 @@ makeinfo OPTIONS."
   (define texinfo-profile
     (profile
      (content (packages->manifest
-               ;; texi2dvi requires various command line tools.
+               ;; texi2any requires various command line tools.
                (list coreutils
                      diffutils
+                     font-arphic-ukai
+                     font-google-noto-serif-cjk-static
                      gawk
                      grep
                      sed
                      tar
-                     texinfo
-                     texlive-base
-                     texlive-bin        ;for GUIX_TEXMF
-                     texlive-epsf
-                     texlive-fonts-ec
-                     texlive-tex-texinfo)))))
+                     texinfo-7
+                     (texlive-local-tree
+                      (list
+                       texlive-epsf
+                       texlive-texinfo
+                       texlive-xetex
+                       texlive-zhspacing)))))))
 
   (define build
     (with-imported-modules '((guix build utils))
@@ -928,9 +1040,13 @@ makeinfo OPTIONS."
                                                 "/lib/locale"))
           (setenv "LC_ALL" "en_US.utf8")
           (setenv "PATH" #+(file-append texinfo-profile "/bin"))
+          (setenv "XDG_DATA_DIRS" #+(file-append texinfo-profile "/share"))
           (setenv "GUIX_TEXMF" #+(file-append texinfo-profile
                                               "/share/texmf-dist"))
 
+          ;; Per Texinfo 7.2 manual, we are meant to use xetex for Chinese.
+          (setenv "PDFTEX" "xetex")
+
           (setvbuf (current-output-port) 'line)
           (setvbuf (current-error-port) 'line)
 
@@ -955,15 +1071,15 @@ makeinfo OPTIONS."
                         (setenv "LANGUAGE" language)
 
 
-                        ;; FIXME: Unfortunately building PDFs for non-Latin
-                        ;; alphabets doesn't work:
-                        ;; <https://lists.gnu.org/archive/html/help-texinfo/2012-01/msg00014.html>.
+                        ;; FIXME: Building PDFs for non-Latin requires a
+                        ;; changed \input line at the beginning.  Unfortunately,
+                        ;; we don't have one for some languages (Russian).
                         (guard (c ((invoke-error? c)
                                    (format (current-error-port)
                                            "~%~%Failed to produce \
 PDF for language '~a'!~%~%"
                                            language)))
-                         (apply invoke #$(file-append texinfo "/bin/makeinfo")
+                         (apply invoke #$(file-append texinfo-7 "/bin/texi2any")
                                 "--pdf" "-o"
                                 (string-append #$output "/"
                                                (normalize language)
@@ -1055,8 +1171,7 @@ must be the Guix top-level source directory, from which PO files are taken."
                  (bindtextdomain "guix-manual"
                                  #+(guix-manual-text-domain source))
                  (bindtextdomain "iso_639-3"      ;language names
-                                 #+(file-append iso-codes
-                                                "/share/locale"))
+                                 #+(file-append iso-codes "/share/locale"))
                  (setenv "LANGUAGE" ,language)
                  (write (gettext ,str ,domain))))
 
@@ -1135,16 +1250,16 @@ must be the Guix top-level source directory, from which PO files are taken."
                     ;; Menu prefetch.
                     (link (@ (rel "prefetch") (href ,(guix-url "menu/index.html"))))
                     ;; Base CSS.
-                    (link (@ (rel "stylesheet") (href ,(guix-url "static/base/css/elements.css"))))
-                    (link (@ (rel "stylesheet") (href ,(guix-url "static/base/css/common.css"))))
-                    (link (@ (rel "stylesheet") (href ,(guix-url "static/base/css/messages.css"))))
-                    (link (@ (rel "stylesheet") (href ,(guix-url "static/base/css/navbar.css"))))
-                    (link (@ (rel "stylesheet") (href ,(guix-url "static/base/css/breadcrumbs.css"))))
-                    (link (@ (rel "stylesheet") (href ,(guix-url "static/base/css/buttons.css"))))
-                    (link (@ (rel "stylesheet") (href ,(guix-url "static/base/css/footer.css"))))
+                    (link (@ (rel "stylesheet") (href ,(guix-url "themes/initial/css/elements.css"))))
+                    (link (@ (rel "stylesheet") (href ,(guix-url "themes/initial/css/common.css"))))
+                    (link (@ (rel "stylesheet") (href ,(guix-url "themes/initial/css/messages.css"))))
+                    (link (@ (rel "stylesheet") (href ,(guix-url "themes/initial/css/navbar.css"))))
+                    (link (@ (rel "stylesheet") (href ,(guix-url "themes/initial/css/breadcrumbs.css"))))
+                    (link (@ (rel "stylesheet") (href ,(guix-url "themes/initial/css/buttons.css"))))
+                    (link (@ (rel "stylesheet") (href ,(guix-url "themes/initial/css/footer.css"))))
 
-                    (link (@ (rel "stylesheet") (href ,(guix-url "static/base/css/page.css"))))
-                    (link (@ (rel "stylesheet") (href ,(guix-url "static/base/css/post.css")))))
+                    (link (@ (rel "stylesheet") (href ,(guix-url "themes/initial/css/page.css"))))
+                    (link (@ (rel "stylesheet") (href ,(guix-url "themes/initial/css/post.css")))))
                    (body
                     (header (@ (class "navbar"))
                             (h1 (a (@ (class "branding")
@@ -1187,7 +1302,7 @@ must be the Guix top-level source directory, from which PO files are taken."
                                                   language))
                                ".html")))
                          "HTML, entirely on one page"))
-                  ,@(if (member language '("ru" "zh_CN"))
+                  ,@(if (member language '("ko" "ru" "ta"))
                         '()
                         `((li (a (@ (href ,(string-append
                                             #$manual
@@ -1293,6 +1408,7 @@ commit date (an integer)."
                          (setenv "PATH"
                                  (string-append #+tar "/bin:"
                                                 #+xz "/bin:"
+                                                #+zstd "/bin:"
                                                 #+texinfo "/bin"))
                          (invoke "tar" "xf" #$(package-source guile))
                          (mkdir-p (string-append #$output "/en/html_node"))
@@ -1356,7 +1472,9 @@ by 'html-identifier-indexes'."
 
 
 (let* ((root (canonicalize-path
-              (string-append (current-source-directory) "/..")))
+              (string-append (or (current-source-directory)
+                                 (string-append (getcwd) "/doc"))
+                             "/..")))
        (commit date (latest-commit+date root))
        (version (or (getenv "GUIX_MANUAL_VERSION")
                     (string-take commit 7)))
@@ -1403,7 +1521,8 @@ by 'html-identifier-indexes'."
     (merge-index-alists guix-split-node-indexes guile-split-node-indexes))
 
   (format (current-error-port)
-          "building manual from work tree around commit ~a, ~a~%"
+          "building manual from work tree (~a) around commit ~a, ~a~%"
+          root
           commit
           (let* ((time (make-time time-utc 0 date))
                  (date (time-utc->date time)))
