@@ -1,7 +1,8 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2015 Joshua S. Grant <jgrant@parenthetical.io>
 ;;; Copyright © 2015 Mark H Weaver <mhw@netris.org>
-;;; Copyright © 2017 Efraim Flashner <efraim@flashner.co.il>
+;;; Copyright © 2017, 2018 Efraim Flashner <efraim@flashner.co.il>
+;;; Copyright © 2020 Marius Bakke <mbakke@fastmail.com>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -20,33 +21,29 @@
 
 (define-module (gnu packages libffcall)
   #:use-module ((guix licenses) #:prefix l:)
-  #:use-module (guix utils) ; string-replace-substring
   #:use-module (guix packages)
   #:use-module (guix download)
-  #:use-module (guix git-download)
   #:use-module (guix build-system gnu))
 
 (define-public libffcall
    (package
     (name "libffcall")
-    (version "1.12")
+    (version "2.5")
     (source
      (origin
-       (method git-fetch)
-       (uri (git-reference
-             (url "https://git.savannah.gnu.org/r/libffcall.git")
-             (commit (string-append
-                       "ffcall-" (string-replace-substring version "." "-")))))
-       (file-name (string-append name "-" version "-checkout"))
+       (method url-fetch)
+       (uri (string-append
+             "mirror://gnu/libffcall/libffcall-" version ".tar.gz"))
        (sha256
         (base32
-         "0p7gg5k4ifcqpmrmdfmr2r4x909cn35g87iff539p6i8891fdfxf"))))
+         "0yb8hk3ksih98249s90lc3nngfs1y4jmi5ckj0wb3604njb20hkz"))))
     (build-system gnu-build-system)
-    (arguments `(#:parallel-build? #f))
+    (arguments
+     '(#:parallel-build? #f
+       #:configure-flags '("--disable-static")))
     (synopsis "Foreign function calls from interpreters")
     (description
      "GNU Libffcall is a collection of libraries that can be used to build
 foreign function call interfaces in embedded interpreters.")
     (home-page "https://www.gnu.org/software/libffcall/")
-    (supported-systems (delete "aarch64-linux" %supported-systems))
     (license l:gpl2+)))
