@@ -1,10 +1,11 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2013 Nikita Karetnikov <nikita@karetnikov.org>
-;;; Copyright © 2016, 2017, 2018 Leo Famulari <leo@famulari.name>
+;;; Copyright © 2016, 2017, 2018, 2023 Leo Famulari <leo@famulari.name>
 ;;; Copyright © 2016 Kei Kebreau <kkebreau@posteo.net>
 ;;; Copyright © 2016, 2019 Efraim Flashner <efraim@flashner.co.il>
 ;;; Copyright © 2018 Rutger Helling <rhelling@mykolab.com>
 ;;; Copyright © 2020 Tobias Geerinckx-Rice <me@tobias.gr>
+;;; Copyright © 2021 Maxim Cournoyer <maxim@guixotic.coop>>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -27,6 +28,7 @@
   #:use-module (gnu packages bdw-gc)
   #:use-module (gnu packages compression)
   #:use-module (gnu packages gtk)
+  #:use-module (gnu packages image)
   #:use-module (gnu packages ncurses)
   #:use-module (gnu packages perl)
   #:use-module (gnu packages pkg-config)
@@ -35,12 +37,16 @@
   #:use-module (gnu packages)
   #:use-module (guix packages)
   #:use-module (guix git-download)
-  #:use-module (guix build-system gnu))
+  #:use-module (guix build-system gnu)
+  #:use-module ((guix search-paths) #:select ($SSL_CERT_DIR $SSL_CERT_FILE)))
 
 (define-public w3m
   (package
     (name "w3m")
-    (version "0.5.3+git20200502")
+    ;; When updating, be careful not to change the derivation of w3m-for-tests,
+    ;; unless you mean to. Changing w3m-for-tests will cause thousands of
+    ;; rebuilds via the reverse dependency graph of xdg-utils.
+    (version "0.5.3+git20230121")
     (source (origin
               (method git-fetch)
               ;; Debian's fork of w3m is the only one that is still maintained.
@@ -50,31 +56,32 @@
               (file-name (git-file-name name version))
               (sha256
                (base32
-                "0yyfhwcwy1dvdbirj6zqwk4gl8z9npfavs0k7ipcg5fd16vnx7mi"))))
+                "0nvhxsqxgxjrr62mvxzhhfzvbvg56g19vlqcgb8mh2x1daazk5ms"))))
     (build-system gnu-build-system)
     (arguments
      '(#:tests? #f  ; no check target
        ;; Use $EDITOR instead of a hard-coded value.
-       #:configure-flags '("--with-editor=")
+       #:configure-flags (list "--with-editor="
+                               "--with-imagelib=imlib2"
+                               "--enable-image=fb,x11")
        #:phases
        (modify-phases %standard-phases
          (add-before 'configure 'fix-perl
            (lambda _ (substitute* '("scripts/w3mmail.cgi.in"
                                     "scripts/dirlist.cgi.in")
-                       (("@PERL@") (which "perl")))
-             #t)))))
+                       (("@PERL@") (which "perl"))))))))
     (inputs
-     `(("gdk-pixbuf" ,gdk-pixbuf)
-       ("libgc" ,libgc)
-       ("libx11" ,libx11)
-       ("ncurses" ,ncurses)
-       ("openssl" ,openssl)
-       ("zlib" ,zlib)))
+     (list gdk-pixbuf
+           imlib2
+           libgc
+           libx11
+           ncurses
+           openssl
+           zlib))
     (native-inputs
-     `(("gettext" ,gettext-minimal)
-       ("perl" ,perl)
-       ("pkg-config" ,pkg-config)))
-    (home-page "http://w3m.sourceforge.net/")
+     (list gettext-minimal perl pkg-config))
+    (native-search-paths (list $SSL_CERT_DIR $SSL_CERT_FILE))
+    (home-page "https://w3m.sourceforge.net/")
     (synopsis "Text-mode web browser")
     (description
      "w3m is a text-based web browser as well as a pager like @code{more} or
@@ -83,3 +90,21 @@ window.  Moreover, w3m can be used as a text formatting tool which
 typesets HTML into plain text.")
     (license (x11-style "file://doc/README"
                         "See 'doc/README' in the distribution."))))
+
+;; Used in the test suite of xdg-utils
+(define-public w3m-for-tests
+  (hidden-package
+   (package
+     (inherit w3m)
+     (name "w3m")
+     (version "0.5.3+git20230121")
+     (source (origin
+               (method git-fetch)
+               ;; Debian's fork of w3m is the only one that is still maintained.
+               (uri (git-reference
+                     (url "https://salsa.debian.org/debian/w3m.git")
+                     (commit (string-append "v" version))))
+               (file-name (git-file-name name version))
+               (sha256
+                (base32
+                 "0nvhxsqxgxjrr62mvxzhhfzvbvg56g19vlqcgb8mh2x1daazk5ms")))))))
