@@ -1,7 +1,7 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2019, 2020 L  p R n  d n <guix@lprndn.info>
 ;;; Copyright © 2020 Ricardo Wurmus <rekado@elephly.net>
-;;; Copyright © 2022 Maxim Cournoyer <maxim.cournoyer@gmail.com>
+;;; Copyright © 2022 Maxim Cournoyer <maxim@guixotic.coop>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -151,7 +151,7 @@ icon themes."
    "The name of the cursor theme to use.")
   (cursor-theme-size
    (number 16)
-   "The size to use for the the cursor theme.")
+   "The size to use for the cursor theme.")
   (allow-debugging?
    maybe-boolean
    "Set to #t to enable debug log level.")
@@ -364,7 +364,7 @@ When unspecified, listen on for any hosts/IP addresses.")
    (boolean #f)
    "Whether a VNC server is started.")
   (vnc-server-command
-   (file-like (file-append tigervnc-server "bin/Xvnc"))
+   (file-like (file-append tigervnc-server "/bin/Xvnc"))
    "The Xvnc command to use for the VNC server, it's possible to provide extra
 options not otherwise exposed along the command, for example to disable
 security:
@@ -397,15 +397,6 @@ When unspecified, listen for any hosts/IP addresses.")
    (list-of-strings '())
    "Extra configuration values to append to the LightDM configuration file."))
 
-(define (lightdm-configuration->greeters-config-dir config)
-  "Return a directory containing all the serialized greeter configurations
-from CONFIG, a <lightdm-configuration> object."
-  (file-union "etc-lightdm"
-              (append-map (lambda (g)
-                            `((,(greeter-configuration->conf-name g)
-                               ,(greeter-configuration->file g))))
-                          (lightdm-configuration-greeters config))))
-
 (define (lightdm-configuration->packages config)
   "Return all the greeter packages and their assets defined in CONFIG, a
 <lightdm-configuration> object, as well as the lightdm package itself."
@@ -472,7 +463,7 @@ enabled=" (if vnc-server? "true" "false") "
 command=" vnc-server-command "
 port=" (number->string vnc-server-port) "\n"
 (if (maybe-value-set? vnc-server-listen-address)
-    (format #f "vnc-server-listen-address=~a" vnc-server-listen-address)
+    (format #f "listen-address=~a" vnc-server-listen-address)
     "") "
 
 #
@@ -496,6 +487,16 @@ port=" (number->string vnc-server-port) "\n"
                                        lightdm-seat-configuration-fields)))
           seats))))
 
+(define (lightdm-configuration-directory config)
+  "Return a directory containing the serialized lightdm configuration
+and all the serialized greeter configurations from CONFIG."
+  (file-union "etc-lightdm"
+              (cons `("lightdm.conf" ,(lightdm-configuration-file config))
+                    (map (lambda (g)
+                           `(,(greeter-configuration->conf-name g)
+                             ,(greeter-configuration->file g)))
+                         (lightdm-configuration-greeters config)))))
+
 (define %lightdm-accounts
   (list (user-group (name "lightdm") (system? #t))
         (user-account
@@ -524,8 +525,7 @@ port=" (number->string vnc-server-port) "\n"
                            (not (= gid (stat:gid st)))))
               (for-each (lambda (file)
                           (chown file uid gid))
-                        (find-files "directory"
-                                    #:directories? #t)))))
+                        (find-files directory #:directories? #t)))))
 
         (when (not (stat "/var/lib/lightdm-data" #f))
           (mkdir-p "/var/lib/lightdm-data"))
@@ -590,9 +590,7 @@ port=" (number->string vnc-server-port) "\n"
                            "/sbin/lightdm")
             #$@(if (lightdm-configuration-debug? config)
                    #~("--debug")
-                   #~())
-            "--config"
-            #$(lightdm-configuration-file config)))
+                   #~())))
 
   (define lightdm-paths
     (let ((lightdm (lightdm-configuration-lightdm config)))
@@ -602,9 +600,6 @@ port=" (number->string vnc-server-port) "\n"
                  '("/bin" "/sbin" "/libexec"))
          ":")))
 
-  (define greeters-config-dir
-    (lightdm-configuration->greeters-config-dir config))
-
   (define data-dirs
     ;; LightDM itself needs to be in XDG_DATA_DIRS for the accountsservice
     ;; interface it provides to be picked up.  The greeters must also be in
@@ -616,30 +611,30 @@ port=" (number->string vnc-server-port) "\n"
   (list
    (shepherd-service
     (documentation "LightDM display manager")
-    (requirement '(dbus-system user-processes host-name))
+    (requirement '(pam dbus-system user-processes host-name))
     (provision '(lightdm display-manager xorg-server))
     (respawn? #f)
     (start
-     #~(lambda ()
-         ;; Note: sadly, environment variables defined for 'lightdm' are
-         ;; cleared and/or overridden by /etc/profile by its spawned greeters,
-         ;; so an out-of-band means such as /etc is required.
-         (fork+exec-command #$lightdm-command
-                            ;; Lightdm needs itself in its PATH.
-                            #:environment-variables
-                            (list
-                             ;; It knows to look for greeter configurations in
-                             ;; XDG_CONFIG_DIRS...
-                             (string-append "XDG_CONFIG_DIRS="
-                                            #$greeters-config-dir)
-                             ;; ... and for greeter .desktop files as well as
-                             ;; lightdm accountsservice interface in
-                             ;; XDG_DATA_DIRS.
-                             (string-append "XDG_DATA_DIRS="
-                                            #$data-dirs)
-                             (string-append "PATH=" #$lightdm-paths)))))
+     ;; Note: sadly, environment variables defined for 'lightdm' are
+     ;; cleared and/or overridden by /etc/profile by its spawned greeters,
+     ;; so an out-of-band means such as /etc is required.
+     #~(make-forkexec-constructor #$lightdm-command
+                                  ;; Lightdm needs itself in its PATH.
+                                  #:environment-variables
+                                  (list
+                                   ;; It looks for greeter .desktop files as
+                                   ;; well as lightdm accountsservice
+                                   ;; interface in XDG_DATA_DIRS.
+                                   (string-append "XDG_DATA_DIRS="
+                                                  #$data-dirs)
+                                   (string-append "PATH=" #$lightdm-paths))))
     (stop #~(make-kill-destructor)))))
 
+(define (lightdm-etc-service config)
+  "Return a list of FILES for @var{etc-service-type} to build the
+/etc/lightdm directory using CONFIG"
+  (list `("lightdm" ,(lightdm-configuration-directory config))))
+
 (define lightdm-service-type
   (handle-xorg-configuration
    lightdm-configuration
@@ -668,13 +663,10 @@ port=" (number->string vnc-server-port) "\n"
            ;; https://github.com/NixOS/nixpkgs/issues/45059.
            (service-extension profile-service-type
                               lightdm-configuration->packages)
-           ;; This is needed for the greeter itself to find its configuration,
-           ;; because XDG_CONF_DIRS gets overridden by /etc/profile.
-           (service-extension
-            etc-service-type
-            (lambda (config)
-              `(("lightdm"
-                 ,(lightdm-configuration->greeters-config-dir config)))))))
+           ;; This is needed for lightdm and greeter
+           ;; to find their configuration
+           (service-extension etc-service-type
+                              lightdm-etc-service)))
     (description "Run @code{lightdm}, the LightDM graphical login manager."))))
 
 
