@@ -1,5 +1,5 @@
 /* GNU Guix --- Functional package management for GNU
-   Copyright (C) 2016, 2017, 2018, 2019 Ludovic Courtès <ludo@gnu.org>
+   Copyright (C) 2016-2019, 2023 Ludovic Courtès <ludo@gnu.org>
 
    This file is part of GNU Guix.
 
@@ -22,6 +22,7 @@
 
 #include <unistd.h>
 #include <cstdlib>
+#include <format>
 
 namespace nix {
 
@@ -53,17 +54,18 @@ static void builtinDownload(const Derivation &drv,
     const string program = settings.guixProgram;
     execv(program.c_str(), (char *const *) argv);
 
-    throw SysError(format("failed to run download program '%1%'") % program);
+    throw SysError(std::format("failed to run download program '{}'", program));
 }
 
 static const std::map<std::string, derivationBuilder> builtins =
 {
-    { "download", builtinDownload }
+    { "download", builtinDownload },
+    { "git-download", builtinDownload }
 };
 
 derivationBuilder lookupBuiltinBuilder(const std::string & name)
 {
-    if (name.substr(0, 8) == "builtin:")
+    if (name.starts_with("builtin:"))
     {
 	auto realName = name.substr(8);
 	auto builder = builtins.find(realName);
