@@ -2,11 +2,9 @@
 
 #include "globals.hh"
 #include "util.hh"
-#include "archive.hh"
 
 #include <map>
-#include <algorithm>
-
+#include <format>
 
 namespace nix {
 
@@ -19,9 +17,6 @@ namespace nix {
 #define DEFAULT_SOCKET_PATH "/daemon-socket/socket"
 
 
-Settings settings;
-
-
 Settings::Settings()
 {
     keepFailed = false;
@@ -32,8 +27,8 @@ Settings::Settings()
     buildCores = 1;
     readOnlyMode = false;
     thisSystem = SYSTEM;
-    maxSilentTime = 0;
-    buildTimeout = 0;
+    maxSilentTime = 3600;
+    buildTimeout = 3600 * 24;
     useBuildHook = true;
     printBuildTrace = false;
     multiplexedBuildOutput = false;
@@ -56,6 +51,8 @@ Settings::Settings()
     envKeepDerivations = false;
     lockCPU = getEnv("NIX_AFFINITY_HACK", "1") == "1";
     showTrace = false;
+    useHostLoopback = true;
+    slirp4netns = SLIRP4NETNS;
 }
 
 
@@ -154,8 +151,8 @@ void Settings::_get(bool & res, const string & name)
     if (i == settings.end()) return;
     if (i->second == "true") res = true;
     else if (i->second == "false") res = false;
-    else throw Error(format("configuration option `%1%' should be either `true' or `false', not `%2%'")
-        % name % i->second);
+    else throw Error(std::format("configuration option `{}' should be either `true' or `false', not `{}'",
+        name, i->second));
 }
 
 
@@ -181,19 +178,19 @@ template<class N> void Settings::_get(N & res, const string & name)
     SettingsMap::iterator i = settings.find(name);
     if (i == settings.end()) return;
     if (!string2Int(i->second, res))
-        throw Error(format("configuration setting `%1%' should have an integer value") % name);
+        throw Error(std::format("configuration setting `{}' should have an integer value", name));
 }
 
 
 string Settings::pack()
 {
     string s;
-    foreach (SettingsMap::iterator, i, settings) {
-        if (i->first.find('\n') != string::npos ||
-            i->first.find('=') != string::npos ||
-            i->second.find('\n') != string::npos)
+    for (auto& i : settings) {
+        if (i.first.find('\n') != string::npos ||
+            i.first.find('=') != string::npos ||
+            i.second.find('\n') != string::npos)
             throw Error("invalid option name/value");
-        s += i->first; s += '='; s += i->second; s += '\n';
+        s += i.first; s += '='; s += i.second; s += '\n';
     }
     return s;
 }
@@ -205,7 +202,4 @@ Settings::SettingsMap Settings::getOverrides()
 }
 
 
-const string nixVersion = PACKAGE_VERSION;
-
-
 }
