@@ -1,8 +1,9 @@
 #include "misc.hh"
+#include <math.h>
 #include "store-api.hh"
 #include "local-store.hh"
-#include "globals.hh"
 
+#include <format>
 
 namespace nix {
 
@@ -28,15 +29,15 @@ void computeFSClosure(StoreAPI & store, const Path & path,
 
         if (includeOutputs) {
             PathSet derivers = store.queryValidDerivers(path);
-            foreach (PathSet::iterator, i, derivers)
-                edges.insert(*i);
+            for (auto& i : derivers)
+                edges.insert(i);
         }
 
         if (includeDerivers && isDerivation(path)) {
             PathSet outputs = store.queryDerivationOutputs(path);
-            foreach (PathSet::iterator, i, outputs)
-                if (store.isValidPath(*i) && store.queryDeriver(*i) == path)
-                    edges.insert(*i);
+            for (auto& i : outputs)
+                if (store.isValidPath(i) && store.queryDeriver(i) == path)
+                    edges.insert(i);
         }
 
     } else {
@@ -44,8 +45,8 @@ void computeFSClosure(StoreAPI & store, const Path & path,
 
         if (includeOutputs && isDerivation(path)) {
             PathSet outputs = store.queryDerivationOutputs(path);
-            foreach (PathSet::iterator, i, outputs)
-                if (store.isValidPath(*i)) edges.insert(*i);
+            for (auto& i : outputs)
+                if (store.isValidPath(i)) edges.insert(i);
         }
 
         if (includeDerivers) {
@@ -54,8 +55,8 @@ void computeFSClosure(StoreAPI & store, const Path & path,
         }
     }
 
-    foreach (PathSet::iterator, i, edges)
-        computeFSClosure(store, *i, paths, flipDirection, includeOutputs, includeDerivers);
+    for (auto& i : edges)
+        computeFSClosure(store, i, paths, flipDirection, includeOutputs, includeDerivers);
 }
 
 
@@ -64,7 +65,7 @@ static void dfsVisit(StoreAPI & store, const PathSet & paths,
     PathSet & parents)
 {
     if (parents.find(path) != parents.end())
-        throw BuildError(format("cycle detected in the references of `%1%'") % path);
+        throw BuildError(std::format("cycle detected in the references of `{}'", path));
 
     if (visited.find(path) != visited.end()) return;
     visited.insert(path);
@@ -74,11 +75,11 @@ static void dfsVisit(StoreAPI & store, const PathSet & paths,
     if (store.isValidPath(path))
         store.queryReferences(path, references);
 
-    foreach (PathSet::iterator, i, references)
+    for (auto& i : references)
         /* Don't traverse into paths that don't exist.  That can
            happen due to substitutes for non-existent paths. */
-        if (*i != path && paths.find(*i) != paths.end())
-            dfsVisit(store, paths, *i, visited, sorted, parents);
+        if (i != path && paths.find(i) != paths.end())
+            dfsVisit(store, paths, i, visited, sorted, parents);
 
     sorted.push_front(path);
     parents.erase(path);
@@ -89,10 +90,30 @@ Paths topoSortPaths(StoreAPI & store, const PathSet & paths)
 {
     Paths sorted;
     PathSet visited, parents;
-    foreach (PathSet::const_iterator, i, paths)
-        dfsVisit(store, paths, *i, visited, sorted, parents);
+    for (const auto& i : paths)
+        dfsVisit(store, paths, i, visited, sorted, parents);
     return sorted;
 }
 
+/* Max of LLONG_MAX is 8 EiB */
+string showBytes(long long bytes)
+{
+    if (llabs(bytes > exp2l(60))) {
+        return std::format("{:7.2f} EiB", bytes / exp2l(60));
+    } else if (llabs(bytes > exp2l(50))) {
+        return std::format("{:7.2f} PiB", bytes / exp2l(50));
+    } else if (llabs(bytes > exp2l(40))) {
+        return std::format("{:7.2f} TiB", bytes / exp2l(40));
+    } else if (llabs(bytes > exp2l(30))) {
+        return std::format("{:7.2f} GiB", bytes / exp2l(30));
+    } else if (llabs(bytes > exp2l(20))) {
+        return std::format("{:7.2f} MiB", bytes / exp2l(20));
+    } else if (llabs(bytes > exp2l(10))) {
+        return std::format("{:7.2f} KiB", bytes / exp2l(10));
+    } else {
+        return std::format("{:4}  bytes", bytes);
+    }
+}
+
 
 }
