@@ -2,6 +2,9 @@
 
 #include <iostream>
 #include <cstring>
+#include <cassert>
+#include <string_view>
+#include <format>
 
 #include "hash.hh"
 #include "archive.hh"
@@ -64,24 +67,27 @@ const string base16Chars = "0123456789abcdef";
 
 string printHash(const Hash & hash)
 {
-    char buf[hash.hashSize * 2];
+    std::vector<char> buf(hash.hashSize * 2);
     for (unsigned int i = 0; i < hash.hashSize; i++) {
         buf[i * 2] = base16Chars[hash.hash[i] >> 4];
         buf[i * 2 + 1] = base16Chars[hash.hash[i] & 0x0f];
     }
-    return string(buf, hash.hashSize * 2);
+    return string(buf.begin(), buf.end());
 }
 
 
-Hash parseHash(HashType ht, const string & s)
+Hash parseHash(HashType ht, std::string_view s)
 {
     Hash hash(ht);
-    if (s.length() != hash.hashSize * 2)
-        throw Error(format("invalid hash `%1%'") % s);
+    if (s.length() != hash.hashSize * 2) {
+	string algo = gcry_md_algo_name(ht);
+        throw Error(std::format("invalid {} hash '{}' ({} bytes but expected {})",
+		    algo, s, (s.length() / 2), hash.hashSize));
+    }
     for (unsigned int i = 0; i < hash.hashSize; i++) {
         string s2(s, i * 2, 2);
         if (!isxdigit(s2[0]) || !isxdigit(s2[1]))
-            throw Error(format("invalid hash `%1%'") % s);
+            throw Error(std::format("invalid hash `{}'", s));
         std::istringstream str(s2);
         int n;
         str >> std::hex >> n;
@@ -97,10 +103,6 @@ unsigned int hashLength32(const Hash & hash)
 }
 
 
-// omitted: E O U T
-const string base32Chars = "0123456789abcdfghijklmnpqrsvwxyz";
-
-
 string printHash32(const Hash & hash)
 {
     Hash hash2(hash);
@@ -129,7 +131,7 @@ string printHash16or32(const Hash & hash)
 }
 
 
-Hash parseHash32(HashType ht, const string & s)
+Hash parseHash32(HashType ht, std::string_view s)
 {
     Hash hash(ht);
     unsigned int len = hashLength32(ht);
@@ -141,7 +143,7 @@ Hash parseHash32(HashType ht, const string & s)
         for (digit = 0; digit < base32Chars.size(); ++digit) /* !!! slow */
             if (base32Chars[digit] == c) break;
         if (digit >= 32)
-            throw Error(format("invalid base-32 hash '%1%'") % s);
+            throw Error(std::format("invalid base-32 hash '{}'", s));
         unsigned int b = n * 5;
         unsigned int i = b / 8;
         unsigned int j = b % 8;
@@ -153,7 +155,7 @@ Hash parseHash32(HashType ht, const string & s)
 }
 
 
-Hash parseHash16or32(HashType ht, const string & s)
+Hash parseHash16or32(HashType ht, std::string_view s)
 {
     Hash hash(ht);
     if (s.size() == hash.hashSize * 2)
@@ -163,13 +165,13 @@ Hash parseHash16or32(HashType ht, const string & s)
         /* base-32 representation */
         hash = parseHash32(ht, s);
     else
-        throw Error(format("hash `%1%' has wrong length for hash type `%2%'")
-            % s % printHashType(ht));
+        throw Error(std::format("hash `{}' has wrong length for hash type `{}'",
+            s, printHashType(ht)));
     return hash;
 }
 
 
-bool isHash(const string & s)
+bool isHash(std::string_view s)
 {
     if (s.length() != 32) return false;
     for (int i = 0; i < 32; i++) {
@@ -244,13 +246,13 @@ Hash hashFile(HashType ht, const Path & path)
     start(ht, ctx);
 
     AutoCloseFD fd = open(path.c_str(), O_RDONLY);
-    if (fd == -1) throw SysError(format("computing hash of file `%1%'") % path);
+    if (fd == -1) throw SysError(std::format("computing hash of file `{}'", path));
 
     unsigned char buf[8192];
     ssize_t n;
     while ((n = read(fd, buf, sizeof(buf)))) {
         checkInterrupt();
-        if (n == -1) throw SysError(format("reading file `%1%'") % path);
+        if (n == -1) throw SysError(std::format("reading file `{}'", path));
         update(ht, ctx, buf, n);
     }
 
