@@ -7,13 +7,15 @@
 #include <cerrno>
 #include <cstdio>
 #include <cstdlib>
-#include <sstream>
 #include <cstring>
+#include <cassert>
+#include <format>
 
 #include <sys/wait.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <limits.h>
+#include <sys/socket.h>
 
 #ifdef __APPLE__
 #include <sys/syscall.h>
@@ -23,6 +25,10 @@
 #include <sys/prctl.h>
 #endif
 
+#ifdef HAVE_LINUX_CLOSE_RANGE_H
+# include <linux/close_range.h>
+#endif
+
 
 extern char * * environ;
 
@@ -30,22 +36,22 @@ extern char * * environ;
 namespace nix {
 
 
-BaseError::BaseError(const FormatOrString & fs, unsigned int status)
+BaseError::BaseError(std::string_view fs, unsigned int status)
     : status(status)
 {
-    err = fs.s;
+    err = fs;
 }
 
 
-BaseError & BaseError::addPrefix(const FormatOrString & fs)
+BaseError & BaseError::addPrefix(std::string fs)
 {
-    prefix_ = fs.s + prefix_;
+    prefix_ = fs + prefix_;
     return *this;
 }
 
 
-SysError::SysError(const FormatOrString & fs)
-    : Error(format("%1%: %2%") % fs.s % strerror(errno))
+SysError::SysError(std::string_view fs)
+    : Error(std::format("{}: {}", fs, strerror(errno)))
     , errNo(errno)
 {
 }
@@ -58,6 +64,27 @@ string getEnv(const string & key, const string & def)
 }
 
 
+string findProgram(const string & program)
+{
+    if(program.empty()) return "";
+
+    if(program[0] == '/') return pathExists(program) ? program : "";
+
+    char *path_ = getenv("PATH");
+    if(path_ == NULL) return "";
+    string path = path_;
+
+    Strings dirs = tokenizeString<Strings>(path, ":");
+    for (const auto& i : dirs) {
+        if(i == "") continue;
+        string f = i + "/" + program;
+        if(pathExists(f)) return f;
+    }
+
+    return "";
+}
+
+
 Path absPath(Path path, Path dir)
 {
     if (path[0] != '/') {
@@ -88,7 +115,7 @@ Path canonPath(const Path & path, bool resolveSymlinks)
     string s;
 
     if (path[0] != '/')
-        throw Error(format("not an absolute path: `%1%'") % path);
+        throw Error(std::format("not an absolute path: `{}'", path));
 
     string::const_iterator i = path.begin(), end = path.end();
     string temp;
@@ -124,7 +151,7 @@ Path canonPath(const Path & path, bool resolveSymlinks)
                the symlink target might contain new symlinks). */
             if (resolveSymlinks && isLink(s)) {
                 if (++followCount >= maxFollow)
-                    throw Error(format("infinite symlink recursion in path `%1%'") % path);
+                    throw Error(std::format("infinite symlink recursion in path `{}'", path));
                 temp = absPath(readLink(s), dirOf(s))
                     + string(i, end);
                 i = temp.begin(); /* restart */
@@ -142,7 +169,7 @@ Path dirOf(const Path & path)
 {
     Path::size_type pos = path.rfind('/');
     if (pos == string::npos)
-        throw Error(format("invalid file name `%1%'") % path);
+        throw Error(std::format("invalid file name `{}'", path));
     return pos == 0 ? "/" : Path(path, 0, pos);
 }
 
@@ -151,7 +178,7 @@ string baseNameOf(const Path & path)
 {
     Path::size_type pos = path.rfind('/');
     if (pos == string::npos)
-        throw Error(format("invalid file name `%1%'") % path);
+        throw Error(std::format("invalid file name `{}'", path));
     return string(path, pos + 1);
 }
 
@@ -169,7 +196,7 @@ struct stat lstat(const Path & path)
 {
     struct stat st;
     if (lstat(path.c_str(), &st))
-        throw SysError(format("getting status of `%1%'") % path);
+        throw SysError(std::format("getting status of `{}'", path));
     return st;
 }
 
@@ -186,7 +213,7 @@ bool pathExists(const Path & path)
 #endif
     if (!res) return true;
     if (errno != ENOENT && errno != ENOTDIR)
-        throw SysError(format("getting status of %1%") % path);
+        throw SysError(std::format("getting status of {}", path));
     return false;
 }
 
@@ -196,15 +223,15 @@ Path readLink(const Path & path)
     checkInterrupt();
     struct stat st = lstat(path);
     if (!S_ISLNK(st.st_mode))
-        throw Error(format("`%1%' is not a symlink") % path);
-    char buf[st.st_size];
-    ssize_t rlsize = readlink(path.c_str(), buf, st.st_size);
+        throw Error(std::format("`{}' is not a symlink", path));
+    std::vector<char> buf(st.st_size);
+    ssize_t rlsize = readlink(path.c_str(), buf.data(), st.st_size);
     if (rlsize == -1)
-        throw SysError(format("reading symbolic link '%1%'") % path);
+        throw SysError(std::format("reading symbolic link '{}'", path));
     else if (rlsize > st.st_size)
-        throw Error(format("symbolic link ‘%1%’ size overflow %2% > %3%")
-            % path % rlsize % st.st_size);
-    return string(buf, st.st_size);
+        throw Error(std::format("symbolic link `{}' size overflow {} > {}",
+            path, rlsize, st.st_size));
+    return string(buf.begin(), buf.end());
 }
 
 
@@ -215,14 +242,11 @@ bool isLink(const Path & path)
 }
 
 
-DirEntries readDirectory(const Path & path)
+static DirEntries readDirectory(DIR *dir)
 {
     DirEntries entries;
     entries.reserve(64);
 
-    AutoCloseDir dir = opendir(path.c_str());
-    if (!dir) throw SysError(format("opening directory `%1%'") % path);
-
     struct dirent * dirent;
     while (errno = 0, dirent = readdir(dir)) { /* sic */
         checkInterrupt();
@@ -230,11 +254,29 @@ DirEntries readDirectory(const Path & path)
         if (name == "." || name == "..") continue;
         entries.emplace_back(name, dirent->d_ino, dirent->d_type);
     }
-    if (errno) throw SysError(format("reading directory `%1%'") % path);
+    if (errno) throw SysError("reading directory");
 
     return entries;
 }
 
+DirEntries readDirectory(const Path & path)
+{
+    AutoCloseDir dir = opendir(path.c_str());
+    if (!dir) throw SysError(std::format("opening directory `{}'", path));
+    return readDirectory(dir);
+}
+
+static DirEntries readDirectory(int fd)
+{
+    /* Since 'closedir' closes the underlying file descriptor, duplicate FD
+       beforehand.  */
+    int fdcopy = dup(fd);
+    if (fdcopy < 0) throw SysError("dup");
+
+    AutoCloseDir dir = fdopendir(fdcopy);
+    if (!dir) throw SysError(std::format("opening directory from file descriptor `{}'", fd));
+    return readDirectory(dir);
+}
 
 unsigned char getFileType(const Path & path)
 {
@@ -252,11 +294,10 @@ string readFile(int fd)
     if (fstat(fd, &st) == -1)
         throw SysError("statting file");
 
-    unsigned char * buf = new unsigned char[st.st_size];
-    AutoDeleteArray<unsigned char> d(buf);
-    readFull(fd, buf, st.st_size);
+    std::vector<unsigned char> buf(st.st_size);
+    readFull(fd, buf.data(), buf.size());
 
-    return string((char *) buf, st.st_size);
+    return string((char *) buf.data(), buf.size());
 }
 
 
@@ -264,7 +305,7 @@ string readFile(const Path & path, bool drain)
 {
     AutoCloseFD fd = open(path.c_str(), O_RDONLY);
     if (fd == -1)
-        throw SysError(format("reading file `%1%'") % path);
+        throw SysError(std::format("reading file `{}'", path));
     return drain ? drainFD(fd) : readFile(fd);
 }
 
@@ -273,7 +314,7 @@ void writeFile(const Path & path, const string & s)
 {
     AutoCloseFD fd = open(path.c_str(), O_WRONLY | O_TRUNC | O_CREAT, 0666);
     if (fd == -1)
-        throw SysError(format("writing file '%1%'") % path);
+        throw SysError(std::format("writing file '{}'", path));
     writeFull(fd, s);
 }
 
@@ -305,47 +346,73 @@ void writeLine(int fd, string s)
 }
 
 
-static void _deletePath(const Path & path, unsigned long long & bytesFreed, size_t linkThreshold)
+static void _deletePathAt(int fd, const Path & path, const Path & fullPath, unsigned long long & bytesFreed, size_t linkThreshold)
 {
     checkInterrupt();
 
-    printMsg(lvlVomit, format("%1%") % path);
+    printMsg(lvlVomit, std::format("{}", fullPath));
 
 #ifdef HAVE_STATX
 # define st_mode stx_mode
 # define st_size stx_size
 # define st_nlink stx_nlink
+#define fstatat(fd, path, stat, flags)           \
+    statx(fd, path, flags, STATX_SIZE | STATX_NLINK | STATX_MODE, stat)
+#define fstat(fd, stat)   \
+    statx(fd, "", AT_EMPTY_PATH, STATX_SIZE | STATX_NLINK | STATX_MODE, stat)
     struct statx st;
-    if (statx(AT_FDCWD, path.c_str(),
-	      AT_SYMLINK_NOFOLLOW,
-	      STATX_SIZE | STATX_NLINK | STATX_MODE, &st) == -1)
-	throw SysError(format("getting status of `%1%'") % path);
 #else
-    struct stat st = lstat(path);
+    struct stat st;
 #endif
+    if (fstatat(fd, path.c_str(), &st, AT_SYMLINK_NOFOLLOW))
+        throw SysError(std::format("getting status of `{}'", fullPath));
 
+    /* Note: if another process modifies what is at 'path' between now and
+       when we actually delete it, this may be inaccurate, but I know of no
+       way to check which file we actually deleted after the fact. */
     if (!S_ISDIR(st.st_mode) && st.st_nlink <= linkThreshold)
 	bytesFreed += st.st_size;
 
     if (S_ISDIR(st.st_mode)) {
-        /* Make the directory writable. */
-        if (!(st.st_mode & S_IWUSR)) {
-            if (chmod(path.c_str(), st.st_mode | S_IWUSR) == -1)
-                throw SysError(format("making `%1%' writable") % path);
-        }
+      /* Note: fds required scales with depth of directory nesting */
+      AutoCloseFD dirfd = openat(fd, path.c_str(),
+                                 O_RDONLY |
+                                 O_DIRECTORY |
+                                 O_NOFOLLOW |
+                                 O_CLOEXEC);
+      if(!dirfd.isOpen())
+        throw SysError(std::format("opening `{}'", fullPath));
 
-        for (auto & i : readDirectory(path))
-            _deletePath(path + "/" + i.name, bytesFreed, linkThreshold);
+      /* st.st_mode may currently be from a different file than what we
+         actually opened, get it straight from the file instead */
+      if(fstat(dirfd, &st))
+        throw SysError(std::format("re-getting status of `{}'", fullPath));
+
+      /* Make the directory writable. */
+      if (!(st.st_mode & S_IWUSR)) {
+        if (fchmod(dirfd, st.st_mode | S_IWUSR) == -1)
+          throw SysError(std::format("making `{}' writable", fullPath));
+      }
+
+      for (auto & i : readDirectory(dirfd))
+        _deletePathAt(dirfd, i.name, path + "/" + i.name, bytesFreed, linkThreshold);
     }
 
     int ret;
-    ret = S_ISDIR(st.st_mode) ? rmdir(path.c_str()) : unlink(path.c_str());
+    ret = unlinkat(fd, path.c_str(), S_ISDIR(st.st_mode) ? AT_REMOVEDIR : 0 );
     if (ret == -1)
-        throw SysError(format("cannot unlink `%1%'") % path);
+        throw SysError(std::format("cannot unlink `{}'", fullPath));
 
 #undef st_mode
 #undef st_size
 #undef st_nlink
+#undef fstatat
+#undef fstat
+}
+
+static void _deletePath(const Path & path, unsigned long long & bytesFreed, size_t linkThreshold)
+{
+  _deletePathAt(AT_FDCWD, path, path, bytesFreed, linkThreshold);
 }
 
 
@@ -359,20 +426,118 @@ void deletePath(const Path & path)
 void deletePath(const Path & path, unsigned long long & bytesFreed, size_t linkThreshold)
 {
     startNest(nest, lvlDebug,
-        format("recursively deleting path `%1%'") % path);
+        std::format("recursively deleting path `{}'", path));
     bytesFreed = 0;
     _deletePath(path, bytesFreed, linkThreshold);
 }
 
+static void copyFile(int sourceFd, int destinationFd)
+{
+    struct stat st;
+    if (fstat(sourceFd, &st) == -1) throw SysError("statting file");
+
+    ssize_t result = copy_file_range(sourceFd, NULL, destinationFd, NULL, st.st_size, 0);
+    if (result < 0 && errno == ENOSYS) {
+	for (size_t remaining = st.st_size; remaining > 0; ) {
+	    unsigned char buf[8192];
+	    size_t count = std::min(remaining, sizeof buf);
+
+	    readFull(sourceFd, buf, count);
+	    writeFull(destinationFd, buf, count);
+	    remaining -= count;
+	}
+    } else {
+	if (result < 0)
+	    throw SysError(std::format("copy_file_range `{}' to `{}'", sourceFd, destinationFd));
+
+	/* If 'copy_file_range' copied less than requested, try again.  */
+	for (ssize_t copied = result; copied < st.st_size; copied += result) {
+	    result = copy_file_range(sourceFd, NULL, destinationFd, NULL,
+				     st.st_size - copied, 0);
+	    if (result < 0)
+		throw SysError(std::format("copy_file_range `{}' to `{}'", sourceFd, destinationFd));
+	}
+    }
+}
+
+static void copyFileRecursively(int sourceroot, const Path &source,
+				int destinationroot, const Path &destination,
+				bool deleteSource)
+{
+    struct stat st;
+    if (fstatat(sourceroot, source.c_str(), &st, AT_SYMLINK_NOFOLLOW) == -1)
+	throw SysError(std::format("statting file `{}'", source));
+
+    if (S_ISREG(st.st_mode)) {
+	AutoCloseFD sourceFd = openat(sourceroot, source.c_str(),
+				      O_CLOEXEC | O_NOFOLLOW | O_RDONLY);
+	if (sourceFd == -1) throw SysError(std::format("opening `{}'", source));
+
+	AutoCloseFD destinationFd = openat(destinationroot, destination.c_str(),
+					   O_CLOEXEC | O_CREAT | O_WRONLY | O_TRUNC
+					   | O_NOFOLLOW | O_EXCL,
+					   st.st_mode);
+	if (destinationFd == -1) throw SysError(std::format("opening `{}'", source));
+
+	copyFile(sourceFd, destinationFd);
+	fchown(destinationFd, st.st_uid, st.st_gid);
+    } else if (S_ISLNK(st.st_mode)) {
+    std::vector<char> target(st.st_size + 1);
+	ssize_t result = readlinkat(sourceroot, source.c_str(), target.data(), st.st_size);
+	if (result != st.st_size) throw SysError("reading symlink target");
+	target[st.st_size] = '\0';
+	int err = symlinkat(target.data(), destinationroot, destination.c_str());
+	if (err != 0)
+	    throw SysError(std::format("creating symlink `{}'", destination));
+	fchownat(destinationroot, destination.c_str(),
+		 st.st_uid, st.st_gid, AT_SYMLINK_NOFOLLOW);
+    } else if (S_ISDIR(st.st_mode)) {
+	int err = mkdirat(destinationroot, destination.c_str(), 0755);
+	if (err != 0)
+	    throw SysError(std::format("creating directory `{}'", destination));
+
+	AutoCloseFD destinationFd = openat(destinationroot, destination.c_str(),
+					   O_CLOEXEC | O_RDONLY | O_DIRECTORY
+					   | O_NOFOLLOW);
+	if (err != 0)
+	    throw SysError(std::format("opening directory `{}'", destination));
+
+	AutoCloseFD sourceFd = openat(sourceroot, source.c_str(),
+				      O_CLOEXEC | O_NOFOLLOW | O_RDONLY);
+	if (sourceFd == -1)
+	    throw SysError(std::format("opening `{}'", source));
+
+        if (deleteSource && !(st.st_mode & S_IWUSR)) {
+	    /* Ensure the directory is writable so files within it can be
+	       deleted.  */
+            if (fchmod(sourceFd, st.st_mode | S_IWUSR) == -1)
+                throw SysError(std::format("making `{}' directory writable", source));
+        }
+
+        for (auto & i : readDirectory(sourceFd))
+	    copyFileRecursively((int)sourceFd, i.name, (int)destinationFd, i.name,
+				deleteSource);
+	fchown(destinationFd, st.st_uid, st.st_gid);
+    } else throw Error(std::format("refusing to copy irregular file `{}'", source));
+
+    if (deleteSource)
+	unlinkat(sourceroot, source.c_str(),
+		 S_ISDIR(st.st_mode) ? AT_REMOVEDIR : 0);
+}
+
+void copyFileRecursively(const Path &source, const Path &destination, bool deleteSource)
+{
+    copyFileRecursively(AT_FDCWD, source, AT_FDCWD, destination, deleteSource);
+}
 
 static Path tempName(Path tmpRoot, const Path & prefix, bool includePid,
     int & counter)
 {
     tmpRoot = canonPath(tmpRoot.empty() ? getEnv("TMPDIR", "/tmp") : tmpRoot, true);
     if (includePid)
-        return (format("%1%/%2%-%3%-%4%") % tmpRoot % prefix % getpid() % counter++).str();
+        return std::format("{}/{}-{}-{}", tmpRoot, prefix, getpid(), counter++);
     else
-        return (format("%1%/%2%-%3%") % tmpRoot % prefix % counter++).str();
+        return std::format("{}/{}-{}", tmpRoot, prefix, counter++);
 }
 
 
@@ -396,11 +561,11 @@ Path createTempDir(const Path & tmpRoot, const Path & prefix,
                "wheel", then "tar" will fail to unpack archives that
                have the setgid bit set on directories. */
             if (chown(tmpDir.c_str(), (uid_t) -1, getegid()) != 0)
-                throw SysError(format("setting group of directory `%1%'") % tmpDir);
+                throw SysError(std::format("setting group of directory `{}'", tmpDir));
             return tmpDir;
         }
         if (errno != EEXIST)
-            throw SysError(format("creating directory `%1%'") % tmpDir);
+            throw SysError(std::format("creating directory `{}'", tmpDir));
     }
 }
 
@@ -414,15 +579,15 @@ Paths createDirs(const Path & path)
     if (lstat(path.c_str(), &st) == -1) {
         created = createDirs(dirOf(path));
         if (mkdir(path.c_str(), 0777) == -1 && errno != EEXIST)
-            throw SysError(format("creating directory `%1%'") % path);
+            throw SysError(std::format("creating directory `{}'", path));
         st = lstat(path);
         created.push_back(path);
     }
 
     if (S_ISLNK(st.st_mode) && stat(path.c_str(), &st) == -1)
-        throw SysError(format("statting symlink `%1%'") % path);
+        throw SysError(std::format("statting symlink `{}'", path));
 
-    if (!S_ISDIR(st.st_mode)) throw Error(format("`%1%' is not a directory") % path);
+    if (!S_ISDIR(st.st_mode)) throw Error(std::format("`{}' is not a directory", path));
 
     return created;
 }
@@ -431,13 +596,10 @@ Paths createDirs(const Path & path)
 void createSymlink(const Path & target, const Path & link)
 {
     if (symlink(target.c_str(), link.c_str()))
-        throw SysError(format("creating symlink from `%1%' to `%2%'") % link % target);
+        throw SysError(std::format("creating symlink from `{}' to `{}'", link, target));
 }
 
 
-LogType logType = ltPretty;
-Verbosity verbosity = lvlInfo;
-
 static int nestingLevel = 0;
 
 
@@ -459,12 +621,12 @@ static string escVerbosity(Verbosity level)
 }
 
 
-void Nest::open(Verbosity level, const FormatOrString & fs)
+void Nest::open(Verbosity level, std::string_view fs)
 {
     if (level <= verbosity) {
         if (logType == ltEscapes)
             std::cerr << "\033[" << escVerbosity(level) << "p"
-                      << fs.s << "\n";
+                      << fs << "\n";
         else
             printMsg_(level, fs);
         nest = true;
@@ -484,7 +646,7 @@ void Nest::close()
 }
 
 
-void printMsg_(Verbosity level, const FormatOrString & fs)
+void printMsg_(Verbosity level, std::string_view fs)
 {
     checkInterrupt();
     if (level > verbosity) return;
@@ -494,15 +656,15 @@ void printMsg_(Verbosity level, const FormatOrString & fs)
             prefix += "|   ";
     else if (logType == ltEscapes && level != lvlInfo)
         prefix = "\033[" + escVerbosity(level) + "s";
-    string s = (format("%1%%2%\n") % prefix % fs.s).str();
+    string s = std::format("{}{}\n", prefix, fs);
     writeToStderr(s);
 }
 
 
-void warnOnce(bool & haveWarned, const FormatOrString & fs)
+void warnOnce(bool & haveWarned, std::string_view fs)
 {
     if (!haveWarned) {
-        printMsg(lvlError, format("warning: %1%") % fs.s);
+        printMsg(lvlError, std::format("warning: {}", fs));
         haveWarned = true;
     }
 }
@@ -521,14 +683,11 @@ void writeToStderr(const string & s)
            write errors in exception handlers to ensure that cleanup
            code runs to completion if the other side of stderr has
            been closed unexpectedly. */
-        if (!std::uncaught_exception()) throw;
+        if (std::uncaught_exceptions() == 0) throw;
     }
 }
 
 
-void (*_writeToStderr) (const unsigned char * buf, size_t count) = 0;
-
-
 void readFull(int fd, unsigned char * buf, size_t count)
 {
     while (count) {
@@ -584,6 +743,17 @@ string drainFD(int fd)
 }
 
 
+/* Wait on FD until MESSAGE has been read.  */
+void waitForMessage(int fd, const string & message)
+{
+    string str(message.length(), '\0');
+    readFull(fd, (unsigned char*)str.data(), message.length());
+    if (str != message)
+	throw Error(std::format("did not receive message '{}' on file descriptor {}",
+	    message, fd));
+}
+
+
 
 //////////////////////////////////////////////////////////////////////
 
@@ -602,7 +772,7 @@ AutoDelete::~AutoDelete()
                 deletePath(path);
             else {
                 if (remove(path.c_str()) == -1)
-                    throw SysError(format("cannot unlink `%1%'") % path);
+                    throw SysError(std::format("cannot unlink `{}'", path));
             }
         }
     } catch (...) {
@@ -673,7 +843,7 @@ void AutoCloseFD::close()
     if (fd != -1) {
         if (::close(fd) == -1)
             /* This should never happen. */
-            throw SysError(format("closing file descriptor %1%") % fd);
+            throw SysError(std::format("closing file descriptor {}", fd));
         fd = -1;
     }
 }
@@ -705,6 +875,67 @@ void Pipe::create()
 }
 
 
+void sendFD(int sock, int fd)
+{
+    ssize_t rc;
+    struct msghdr msg;
+    struct cmsghdr *cmsg;
+    char cmsgbuf[CMSG_SPACE(sizeof(fd))];
+    struct iovec iov;
+    char dummy = '\0';
+    memset(&msg, 0, sizeof(msg));
+    iov.iov_base = &dummy;
+    iov.iov_len = 1;
+    msg.msg_iov = &iov;
+    msg.msg_iovlen = 1;
+    msg.msg_control = cmsgbuf;
+    msg.msg_controllen = sizeof(cmsgbuf);
+    cmsg = CMSG_FIRSTHDR(&msg);
+    cmsg->cmsg_level = SOL_SOCKET;
+    cmsg->cmsg_type = SCM_RIGHTS;
+    cmsg->cmsg_len = CMSG_LEN(sizeof(fd));
+    memcpy(CMSG_DATA(cmsg), &fd, sizeof(fd));
+    msg.msg_controllen = cmsg->cmsg_len;
+    do
+    {
+        rc = sendmsg(sock, &msg, 0);
+    } while(rc < 0 && errno == EINTR);
+    if(rc < 0)
+        throw SysError("sending fd");
+}
+
+
+int receiveFD(int sock)
+{
+    int fd;
+    ssize_t rc;
+    struct msghdr msg;
+    struct cmsghdr *cmsg;
+    char cmsgbuf[CMSG_SPACE(sizeof(fd))];
+    struct iovec iov;
+    char dummy = '\0';
+    memset(&msg, 0, sizeof(msg));
+    iov.iov_base = &dummy;
+    iov.iov_len = 1;
+    msg.msg_iov = &iov;
+    msg.msg_iovlen = 1;
+    msg.msg_control = cmsgbuf;
+    msg.msg_controllen = sizeof(cmsgbuf);
+    do
+    {
+        rc = recvmsg(sock, &msg, 0);
+    } while(rc < 0 && errno == EINTR);
+    if (rc < 0)
+        throw SysError("receiving fd");
+    if (rc == 0)
+        throw Error("received EOF (empty message) while receiving fd");
+
+    cmsg = CMSG_FIRSTHDR(&msg);
+    if (cmsg == NULL || cmsg->cmsg_type != SCM_RIGHTS)
+        throw Error("received message without an fd");
+    memcpy(&fd, CMSG_DATA(cmsg), sizeof(fd));
+    return fd;
+}
 
 //////////////////////////////////////////////////////////////////////
 
@@ -788,13 +1019,13 @@ void Pid::kill(bool quiet)
     if (pid == -1 || pid == 0) return;
 
     if (!quiet)
-        printMsg(lvlError, format("killing process %1%") % pid);
+        printMsg(lvlError, std::format("killing process {}", pid));
 
     /* Send the requested signal to the child.  If it has its own
        process group, send the signal to every process in the child
        process group (which hopefully includes *all* its children). */
     if (::kill(separatePG ? -pid : pid, killSignal) != 0)
-        printMsg(lvlError, (SysError(format("killing process %1%") % pid).msg()));
+        printMsg(lvlError, (SysError(std::format("killing process {}", pid)).msg()));
 
     /* Wait until the child dies, disregarding the exit status. */
     int status;
@@ -802,7 +1033,7 @@ void Pid::kill(bool quiet)
         checkInterrupt();
         if (errno != EINTR) {
             printMsg(lvlError,
-                (SysError(format("waiting for process %1%") % pid).msg()));
+                (SysError(std::format("waiting for process {}", pid)).msg()));
             break;
         }
     }
@@ -843,7 +1074,7 @@ void Pid::setKillSignal(int signal)
 
 void killUser(uid_t uid)
 {
-    debug(format("killing all processes running under uid `%1%'") % uid);
+    debug(std::format("killing all processes running under uid `{}'", uid));
 
     assert(uid != 0); /* just to be safe... */
 
@@ -873,7 +1104,7 @@ void killUser(uid_t uid)
 #endif
             if (errno == ESRCH) break; /* no more processes */
             if (errno != EINTR)
-                throw SysError(format("cannot kill processes for uid `%1%'") % uid);
+                throw SysError(std::format("cannot kill processes for uid `{}'", uid));
         }
 
         _exit(0);
@@ -885,7 +1116,7 @@ void killUser(uid_t uid)
     if (status == SIGKILL) return;
 #endif
     if (status != 0)
-        throw Error(format("cannot kill processes for uid `%1%': %2%") % uid % statusToString(status));
+        throw Error(std::format("cannot kill processes for uid `{}': {}", uid, statusToString(status)));
 
     /* !!! We should really do some check to make sure that there are
        no processes left running under `uid', but there is no portable
@@ -957,7 +1188,9 @@ string runProgram(Path program, bool searchPath, const Strings & args)
         else
             execv(program.c_str(), stringsToCharPtrs(args_).data());
 
-        throw SysError(format("executing `%1%'") % program);
+        int err = errno;
+        printMsg(lvlError, std::format("executing `{}': {}", program, strerror(err)));
+        _exit(127);
     });
 
     pipe.writeSide.close();
@@ -967,8 +1200,8 @@ string runProgram(Path program, bool searchPath, const Strings & args)
     /* Wait for the child to finish. */
     int status = pid.wait(true);
     if (!statusOk(status))
-        throw ExecError(format("program `%1%' %2%")
-            % program % statusToString(status));
+        throw ExecError(std::format("program `{}' {}",
+            program, statusToString(status)));
 
     return result;
 }
@@ -976,12 +1209,19 @@ string runProgram(Path program, bool searchPath, const Strings & args)
 
 void closeMostFDs(const set<int> & exceptions)
 {
-    int maxFD = 0;
-    maxFD = sysconf(_SC_OPEN_MAX);
-    for (int fd = 0; fd < maxFD; ++fd)
-        if (fd != STDIN_FILENO && fd != STDOUT_FILENO && fd != STDERR_FILENO
-            && exceptions.find(fd) == exceptions.end())
-            close(fd); /* ignore result */
+#ifdef HAVE_CLOSE_RANGE
+    if (exceptions.empty())
+	 close_range(3, ~0U, 0);
+    else
+#endif
+    {
+	 int maxFD = 0;
+	 maxFD = sysconf(_SC_OPEN_MAX);
+	 for (int fd = 0; fd < maxFD; ++fd)
+	      if (fd != STDIN_FILENO && fd != STDOUT_FILENO && fd != STDERR_FILENO
+		  && exceptions.find(fd) == exceptions.end())
+		   close(fd); /* ignore result */
+    }
 }
 
 
@@ -993,18 +1233,24 @@ void closeOnExec(int fd)
         throw SysError("setting close-on-exec flag");
 }
 
+void keepOnExec(int fd)
+{
+    int prev;
+    if ((prev = fcntl(fd, F_GETFD, 0)) == -1 ||
+        fcntl(fd, F_SETFD, prev & ~FD_CLOEXEC) == -1)
+        throw SysError("clearing close-on-exec flag");
+}
 
 //////////////////////////////////////////////////////////////////////
 
 
-volatile sig_atomic_t _isInterrupted = 0;
 
 void _interrupted()
 {
     /* Block user interrupts while an exception is being handled.
        Throwing an exception while another exception is being handled
        kills the program! */
-    if (!std::uncaught_exception()) {
+    if (std::uncaught_exceptions() == 0) {
         _isInterrupted = 0;
         throw Interrupted("interrupted by the user");
     }
@@ -1037,9 +1283,9 @@ template vector<string> tokenizeString(const string & s, const string & separato
 string concatStringsSep(const string & sep, const Strings & ss)
 {
     string s;
-    foreach (Strings::const_iterator, i, ss) {
+    for (const auto& i : ss) {
         if (s.size() != 0) s += sep;
-        s += *i;
+        s += i;
     }
     return s;
 }
@@ -1048,9 +1294,9 @@ string concatStringsSep(const string & sep, const Strings & ss)
 string concatStringsSep(const string & sep, const StringSet & ss)
 {
     string s;
-    foreach (StringSet::const_iterator, i, ss) {
+    for (const auto& i : ss) {
         if (s.size() != 0) s += sep;
-        s += *i;
+        s += i;
     }
     return s;
 }
@@ -1067,14 +1313,14 @@ string statusToString(int status)
 {
     if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
         if (WIFEXITED(status))
-            return (format("failed with exit code %1%") % WEXITSTATUS(status)).str();
+            return std::format("failed with exit code {}", WEXITSTATUS(status));
         else if (WIFSIGNALED(status)) {
             int sig = WTERMSIG(status);
 #if HAVE_STRSIGNAL
             const char * description = strsignal(sig);
-            return (format("failed due to signal %1% (%2%)") % sig % description).str();
+            return std::format("failed due to signal {} ({})", sig, description);
 #else
-            return (format("failed due to signal %1%") % sig).str();
+            return std::format("failed due to signal {}", sig);
 #endif
         }
         else
@@ -1089,18 +1335,12 @@ bool statusOk(int status)
 }
 
 
-bool hasSuffix(const string & s, const string & suffix)
+void expect(std::istream & str, std::string_view s)
 {
-    return s.size() >= suffix.size() && string(s, s.size() - suffix.size()) == suffix;
-}
-
-
-void expect(std::istream & str, const string & s)
-{
-    char s2[s.size()];
-    str.read(s2, s.size());
-    if (string(s2, s.size()) != s)
-        throw FormatError(format("expected string `%1%'") % s);
+    std::vector<char> s2(s.size());
+    str.read(s2.data(), s2.size());
+    if (string(s2.begin(), s2.end()) != s)
+        throw FormatError(std::format("expected string `{}'", s));
 }
 
 
@@ -1135,13 +1375,31 @@ bool endOfList(std::istream & str)
     return false;
 }
 
+string decodeOctalEscaped(const string & s)
+{
+    string r;
+    for (string::const_iterator i = s.begin(); i != s.end(); ) {
+        if (*i != '\\') { r += *(i++); continue; }
+        unsigned char c = 0;
+        ++i;
+        for(int j = 0; j < 3; j++) {
+          if(i == s.end() || *i < '0' || *i >= '8')
+            throw Error("malformed octal escape");
+          c = c * 8 + (*i - '0');
+          ++i;
+        }
+        r += c;
+    }
+    return r;
+}
+
 
 void ignoreException()
 {
     try {
         throw;
     } catch (std::exception & e) {
-        printMsg(lvlError, format("error (ignored): %1%") % e.what());
+        printMsg(lvlError, std::format("error (ignored): {}", e.what()));
     }
 }
 
@@ -1155,7 +1413,10 @@ void commonChildInit(Pipe & logPipe)
        that e.g. ssh cannot open /dev/tty) and it doesn't receive
        terminal signals. */
     if (setsid() == -1)
-        throw SysError(format("creating a new session"));
+        throw SysError("creating a new session");
+
+    /* Close the read end so only the parent holds a reference to it.  */
+    logPipe.readSide.close();
 
     /* Dup the write side of the logger pipe into stderr. */
     if (dup2(logPipe.writeSide, STDERR_FILENO) == -1)
@@ -1168,7 +1429,7 @@ void commonChildInit(Pipe & logPipe)
     /* Reroute stdin to /dev/null. */
     int fdDevNull = open(pathNullDevice.c_str(), O_RDWR);
     if (fdDevNull == -1)
-        throw SysError(format("cannot open `%1%'") % pathNullDevice);
+        throw SysError(std::format("cannot open `{}'", pathNullDevice));
     if (dup2(fdDevNull, STDIN_FILENO) == -1)
         throw SysError("cannot dup null device into stdin");
     close(fdDevNull);
@@ -1178,7 +1439,7 @@ void commonChildInit(Pipe & logPipe)
 
 Agent::Agent(const string &command, const Strings &args, const std::map<string, string> &env)
 {
-    debug(format("starting agent '%1%'") % command);
+    debug(std::format("starting agent '{}'", command));
 
     /* Create a pipe to get the output of the child. */
     fromAgent.create();
@@ -1214,7 +1475,7 @@ Agent::Agent(const string &command, const Strings &args, const std::map<string,
 
         execv(command.c_str(), stringsToCharPtrs(allArgs).data());
 
-        throw SysError(format("executing `%1%'") % command);
+        throw SysError(std::format("executing `{}'", command));
     });
 
     pid.setSeparatePG(true);
