@@ -17,12 +17,12 @@
    You should have received a copy of the GNU General Public License
    along with GNU Guix.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include <config.h>
+#include "config.h"
 
-#include <types.hh>
-#include "shared.hh"
-#include <globals.hh>
-#include <util.hh>
+#include "types.hh"
+#include "nix-daemon.hh"
+#include "globals.hh"
+#include "util.hh"
 
 #include <gcrypt.h>
 
@@ -36,19 +36,13 @@
 #include <netdb.h>
 #include <strings.h>
 #include <exception>
-#include <iostream>
+#include <format>
 
 #include <libintl.h>
 #include <locale.h>
 
-/* Variables used by `nix-daemon.cc'.  */
-volatile ::sig_atomic_t blockInt;
-char **argvSaved;
-
 using namespace nix;
 
-/* Entry point in `nix-daemon.cc'.  */
-extern void run (const std::vector<int> &);
 
 
 /* Command-line options.  */
@@ -90,6 +84,7 @@ builds derivations on behalf of its clients.");
 #define GUIX_OPT_MAX_SILENT_TIME 19
 #define GUIX_OPT_LOG_COMPRESSION 20
 #define GUIX_OPT_DISCOVER 21
+#define GUIX_OPT_ISOLATE_HOST_LOOPBACK 22
 
 static const struct argp_option options[] =
   {
@@ -160,6 +155,8 @@ to live outputs") },
       n_("listen for connections on SOCKET") },
     { "debug", GUIX_OPT_DEBUG, 0, 0,
       n_("produce debugging output") },
+    { "isolate-host-loopback", GUIX_OPT_ISOLATE_HOST_LOOPBACK, 0, 0,
+      n_("do not allow fixed-output chroot builds to access the host loopback") },
     { 0, 0, 0, 0, 0 }
   };
 
@@ -184,7 +181,7 @@ string_to_bool (const char *arg, bool dflt = true)
   else if (strcasecmp (arg, "no") == 0)
     return false;
   else
-    throw nix::Error (format ("'%1%': invalid Boolean value") % arg);
+    throw nix::Error(std::format("'{}': invalid Boolean value", arg));
 }
 
 /* Parse a single option. */
@@ -198,10 +195,8 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
     case GUIX_OPT_CHROOT_DIR:
       {
-	std::string chroot_dirs;
+	std::string chroot_dirs {settings.get("build-extra-chroot-dirs", "")};
 
-	chroot_dirs = settings.get ("build-extra-chroot-dirs",
-				    (std::string) "");
 	if (chroot_dirs == "")
 	  chroot_dirs = arg;
 	else
@@ -294,6 +289,9 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case GUIX_OPT_SYSTEM:
       settings.thisSystem = arg;
       break;
+    case GUIX_OPT_ISOLATE_HOST_LOOPBACK:
+      settings.useHostLoopback = false;
+      break;
     default:
       return (error_t) ARGP_ERR_UNKNOWN;
     }
@@ -331,7 +329,8 @@ open_unix_domain_socket (const char *file)
   struct sockaddr_un addr;
   addr.sun_family = AF_UNIX;
   if (fileRel.size () >= sizeof (addr.sun_path))
-    throw Error (format (_("socket file name '%1%' is too long")) % fileRel);
+    throw Error (std::vformat( (_("socket file name '%1%' is too long")), 
+      std::make_format_args(fileRel)));
   strcpy (addr.sun_path, fileRel.c_str ());
 
   unlink (file);
@@ -343,13 +342,15 @@ open_unix_domain_socket (const char *file)
   int res = bind (fdSocket, (struct sockaddr *) &addr, sizeof addr);
   umask (oldMode);
   if (res == -1)
-    throw SysError (format (_("cannot bind to socket '%1%'")) % file);
+    throw SysError (std::vformat( (_("cannot bind to socket '{}'")),
+      std::make_format_args(file)));
 
   if (chdir ("/") == -1) /* back to the root */
     throw SysError (_("cannot change current directory"));
 
   if (listen (fdSocket, 5) == -1)
-    throw SysError (format (_("cannot listen on socket '%1%'")) % file);
+    throw SysError (std::vformat( (_("cannot listen on socket '{}'")),
+      std::make_format_args(file)));
 
   return fdSocket.borrow ();
 }
@@ -367,7 +368,7 @@ open_inet_socket (const struct sockaddr *address, socklen_t length)
     throw SysError (_("cannot bind TCP socket"));
 
   if (listen (fd, 5) == -1)
-    throw SysError (format (_("cannot listen on TCP socket")));
+    throw SysError (_("cannot listen on TCP socket"));
 
   return fd.borrow ();
 }
@@ -417,11 +418,9 @@ listening_sockets (const std::list<std::string> &options)
 				 &hints, &res);
 
 	  if (err != 0)
-	    throw Error(format ("failed to look up '%1%': %2%")
-			% option % gai_strerror (err));
+	    throw Error(std::format("failed to look up '{}': {}", option, gai_strerror(err)));
 
-	  printMsg (lvlDebug, format ("listening on '%1%', port '%2%'")
-		    % host % port);
+	  printMsg(lvlDebug, std::format("listening on '{}', port '{}'", host, port));
 
 	  /* XXX: Pick the first result, RES.  */
 	  result.push_back (open_inet_socket (res->ai_addr,
@@ -524,17 +523,19 @@ main (int argc, char *argv[])
 	/* We were not "socket-activated" so open the sockets specified by
 	   LISTEN_OPTIONS.  */
 	sockets = listening_sockets (listen_options);
-      else
-	printMsg (lvlInfo,
-		  format (ngettext ("socket-activated with %1% socket",
-				    "socket-activated with %1% sockets",
-				    sockets.size ()))
-		  % sockets.size ());
+      else {
+        auto size = sockets.size();
+	      printMsg (lvlInfo,
+		      std::vformat((ngettext ("socket-activated with %1% socket",
+				                  "socket-activated with %1% sockets",
+				                  size)),
+		        std::make_format_args(size)));
+      }
 
       /* Effect all the changes made via 'settings.set'.  */
       settings.update ();
       printMsg(lvlDebug,
-	       format ("build log compression: %1%") % settings.logCompression);
+	      std::format("build log compression: {}", int(settings.logCompression)));
 
       if (geteuid () == 0 && settings.buildUsersGroup.empty ())
 	fprintf (stderr, _("warning: daemon is running as root, so \
@@ -547,7 +548,7 @@ using `--build-users-group' is highly recommended\n"));
 	  chroot_dirs = settings.get ("build-extra-chroot-dirs",
 				      (std::string) "");
 	  printMsg (lvlDebug,
-		    format ("extra chroot directories: '%1%'") % chroot_dirs);
+		  std::format("extra chroot directories: '{}'", chroot_dirs));
 	}
 
       if (useDiscover)
@@ -562,9 +563,8 @@ using `--build-users-group' is highly recommended\n"));
         });
       }
 
-      printMsg (lvlDebug,
-		format ("automatic deduplication set to %1%")
-		% settings.autoOptimiseStore);
+      printMsg(lvlDebug,
+		    std::format("automatic deduplication set to {}", settings.autoOptimiseStore));
 
       run (sockets);
     }
