@@ -1,5 +1,5 @@
 #include "config.h"
-#include "shared.hh"
+#include "nix-daemon.hh"
 #include "local-store.hh"
 #include "util.hh"
 #include "serialise.hh"
@@ -10,8 +10,10 @@
 #include "builtins.hh"
 
 #include <algorithm>
+#include <format>
 
 #include <cstring>
+#include <cassert>
 #include <unistd.h>
 #include <signal.h>
 #include <sys/types.h>
@@ -165,6 +167,24 @@ static void setSigPollAction(bool enable)
 #endif
 }
 
+static void handleSignal(int signum)
+{
+    string name = program_invocation_short_name;
+    auto message = name + ": PID " + std::to_string(getpid())
+	+ " caught signal " + std::to_string(signum) + "\n";
+    writeFull(STDERR_FILENO, (unsigned char *) message.c_str(), message.length());
+    _isInterrupted = 1;
+    blockInt = 1;
+}
+
+static void setTerminationSignalHandler()
+{
+    auto signals = { SIGINT, SIGTERM, SIGHUP };
+    for (int signum: signals) {
+	signal(signum, handleSignal);
+    }
+}
+
 
 /* startWork() means that we're starting an operation for which we
    want to send out stderr to the client. */
@@ -318,7 +338,7 @@ static void performOp(bool trusted, unsigned int clientVersion,
     case wopHasSubstitutes: {
         Path path = readStorePath(from);
         startWork();
-        PathSet res = store->querySubstitutablePaths(singleton<PathSet>(path));
+        PathSet res = store->querySubstitutablePaths(PathSet{path});
         stopWork();
         writeInt(res.find(path) != res.end(), to);
         break;
@@ -622,7 +642,7 @@ static void performOp(bool trusted, unsigned int clientVersion,
                     if (pw != NULL)
                         store->createUser(value, pw->pw_uid);
                     else
-                        printMsg(lvlInfo, format("user name %1% not found") % value);
+                        printMsg(lvlInfo, std::format("user name {} not found", value));
 		}
                 else
                     settings.set(trusted ? name : "untrusted-" + name, value);
@@ -638,7 +658,7 @@ static void performOp(bool trusted, unsigned int clientVersion,
         Path path = absPath(readString(from));
         startWork();
         SubstitutablePathInfos infos;
-        store->querySubstitutablePathInfos(singleton<PathSet>(path), infos);
+        store->querySubstitutablePathInfos(PathSet{path}, infos);
         stopWork();
         SubstitutablePathInfos::iterator i = infos.find(path);
         if (i == infos.end())
@@ -661,12 +681,12 @@ static void performOp(bool trusted, unsigned int clientVersion,
         store->querySubstitutablePathInfos(paths, infos);
         stopWork();
         writeInt(infos.size(), to);
-        foreach (SubstitutablePathInfos::iterator, i, infos) {
-            writeString(i->first, to);
-            writeString(i->second.deriver, to);
-            writeStrings(i->second.references, to);
-            writeLongLong(i->second.downloadSize, to);
-            writeLongLong(i->second.narSize, to);
+        for (auto& i : infos) {
+            writeString(i.first, to);
+            writeString(i.second.deriver, to);
+            writeStrings(i.second.references, to);
+            writeLongLong(i.second.downloadSize, to);
+            writeLongLong(i.second.narSize, to);
         }
         break;
     }
@@ -736,8 +756,25 @@ static void performOp(bool trusted, unsigned int clientVersion,
 	break;
     }
 
+    case wopSubstituteURLs: {
+	startWork();
+	Strings urls;
+	if (settings.get("build-use-substitutes", std::string("false")) == "true") {
+	    /* First check the client-provided substitute URLs, then those
+	       passed to the daemon.  */
+	    auto str = settings.get("untrusted-substitute-urls",  std::string(""));
+	    if (str.empty()) {
+		str = settings.get("substitute-urls",  std::string(""));
+	    }
+	    urls = tokenizeString<Strings>(str);
+	}
+	stopWork();
+	writeStrings(urls, to);
+	break;
+    }
+
     default:
-        throw Error(format("invalid operation %1%") % op);
+        throw Error(std::format("invalid operation {}", op));
     }
 }
 
@@ -786,6 +823,10 @@ static void processConnection(bool trusted, uid_t userId)
             throw Error("if you run `nix-daemon' as root, then you MUST set `build-users-group'!");
 #endif
 
+	/* Catch SIGTERM & co. to ensure proper termination: closing the store
+	   and its database, thereby deleting its WAL file.  */
+	setTerminationSignalHandler();
+
         /* Open the store. */
         store = std::shared_ptr<StoreAPI>(new LocalStore(reserveSpace));
 
@@ -795,7 +836,7 @@ static void processConnection(bool trusted, uid_t userId)
             if (pw != NULL && pw->pw_name != NULL)
                 store->createUser(pw->pw_name, userId);
             else
-                printMsg(lvlInfo, format("user with UID %1% not found") % userId);
+                printMsg(lvlInfo, std::format("user with UID {} not found", userId));
 	}
 
         stopWork();
@@ -843,7 +884,7 @@ static void processConnection(bool trusted, uid_t userId)
 
     canSendStderr = false;
     _isInterrupted = false;
-    printMsg(lvlDebug, format("%1% operations") % opCount);
+    printMsg(lvlDebug, std::format("{} operations", opCount));
 }
 
 
@@ -924,14 +965,16 @@ static void acceptConnection(int fdSocket)
 	    clientPid = cred.pid;
 	    clientUid = cred.uid;
 	    clientGid = cred.gid;
-	    trusted = clientUid == 0;
+
+	    /* The root user is always trusted; additionally, when running as
+	       an unprivileged user, that user is also trusted.  */
+	    trusted = (clientUid == 0) || (clientUid == getuid());
 
             struct passwd * pw = getpwuid(cred.uid);
             string user = pw ? pw->pw_name : std::to_string(cred.uid);
 
-	    printMsg(lvlInfo,
-		     format((string) "accepted connection from pid %1%, user %2%")
-		     % clientPid % user);
+	    printMsg(lvlInfo, std::format("accepted connection from pid {}, user {}",
+		    clientPid, user));
 #endif
 	} else {
 	    char address_str[128];
@@ -950,9 +993,7 @@ static void acceptConnection(int fdSocket)
 	    }
 
 	    if (result != NULL) {
-		printMsg(lvlInfo,
-			 format("accepted connection from %1%")
-			 % address_str);
+		printMsg(lvlInfo, std::format("accepted connection from {}", address_str));
 	    }
 	}
 
@@ -962,7 +1003,7 @@ static void acceptConnection(int fdSocket)
 
                 /* Background the daemon. */
                 if (setsid() == -1)
-                    throw SysError(format("creating a new session"));
+                    throw SysError("creating a new session");
 
                 /* Restore normal handling of SIGCHLD. */
                 setSigChldAction(false);
@@ -991,7 +1032,7 @@ static void acceptConnection(int fdSocket)
     } catch (Interrupted & e) {
 	throw;
     } catch (Error & e) {
-	printMsg(lvlError, format("error processing connection: %1%") % e.msg());
+	printMsg(lvlError, std::format("error processing connection: {}", e.msg()));
     }
 }
 
@@ -1030,7 +1071,7 @@ static void daemonLoop(const std::vector<int>& sockets)
 	    int err = errno;
 	    if (err == EINTR)
 		continue;
-	    throw SysError(format("select error: %1%") % strerror(err));
+	    throw SysError(std::format("select error: {}", strerror(err)));
 	}
 
 	for (unsigned int i = 0; i < sockets.size(); i++) {
