@@ -1,6 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2018 Ricardo Wurmus <rekado@elephly.net>
-;;; Copyright © 2019, 2020, 2022 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2019-2020, 2022, 2024 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -50,7 +50,7 @@
   #:use-module (ice-9 match))
 
 (define (gpg+git-available?)
-  (and (which (git-command))
+  (and #t                                         ;'git' is always available
        (which (gpg-command)) (which (gpgconf-command))))
 
 (define commit-id-string
@@ -154,7 +154,7 @@
                    (url "test")))
          (test-dir (channel-instance-checkout instance--simple)))
     (mock ((guix git) update-cached-checkout
-           (lambda* (url #:key ref starting-commit)
+           (lambda* (url #:key ref starting-commit verify-certificate?)
              (match url
                ("test" (values test-dir "caf3cabba9e" #f))
                (_      (values (channel-instance-checkout instance--no-deps)
@@ -173,7 +173,7 @@
                    (url "test")))
          (test-dir (channel-instance-checkout instance--with-dupes)))
     (mock ((guix git) update-cached-checkout
-           (lambda* (url #:key ref starting-commit)
+           (lambda* (url #:key ref starting-commit verify-certificate?)
              (match url
                ("test" (values test-dir "caf3cabba9e" #f))
                (_      (values (channel-instance-checkout instance--no-deps)
@@ -191,12 +191,60 @@
                            (and (eq? (channel-name
                                       (channel-instance-channel instance))
                                      'test-channel)
-                                (string=? (channel-commit
-                                           (channel-instance-channel instance))
-                                          "abc1234")))
+                                (equal? (channel-commit
+                                         (channel-instance-channel instance))
+                                        "abc1234")))
                          instances)))))))
 
-(unless (which (git-command)) (test-skip 1))
+(test-equal "latest-channel-instances reads dependencies from most-specific instance"
+  '(chan1 chan2)
+  ;; Here '.guix-channel' in DIRECTORY2 is less specific than the
+  ;; user-provided channel spec in ONE: the latter specifies a commit.  Since
+  ;; the most specific one "wins", the bogus '.guix-channel' file added in
+  ;; DIRECTORY1 as its second commit must not be taken into account.
+  ;; See <https://issues.guix.gnu.org/68822>.
+  (with-temporary-git-repository directory1
+      `((add "a.scm" "(define-module (a))")
+        (commit "first commit")
+        (add ".guix-channel"
+             ,(object->string
+               '(channel
+                 (version 0)
+                 (dependencies
+                  ;; Attempting to fetch this dependency would fail.
+                  (channel
+                   (name nonexistent-dependency)
+                   (url "http://guix.example.org/does-not-exist.git"))))))
+        (commit "second commit"))
+    (with-temporary-git-repository directory2
+        `((add ".guix-channel"
+               ,(object->string
+                 `(channel (version 0)
+                           (dependencies
+                            (channel
+                             (name chan1)
+                             ;; Note: no 'commit' field here.
+                             (url ,(string-append "file://" directory1)))))))
+          (commit "initial commit"))
+      (with-repository directory1 repository
+        (let* ((commit (find-commit repository "first"))
+               (one    (channel
+                        (url (string-append "file://" directory1))
+                        (commit (oid->string (commit-id commit))) ;<- specific
+                        (name 'chan1)))
+               (two    (channel
+                        (url (string-append "file://" directory2))
+                        (name 'chan2))))
+
+          (with-store store
+            (map (compose channel-name channel-instance-channel)
+                 (delete-duplicates
+                  (append (latest-channel-instances store (list one two))
+                          (latest-channel-instances store (list two one)))
+                  (lambda (instance1 instance2)
+                    (string=? (channel-instance-commit instance1)
+                              (channel-instance-commit instance2)))))))))))
+
 (test-equal "latest-channel-instances #:validate-pull"
   'descendant
 
@@ -306,7 +354,6 @@
                (depends? drv3
                          (list drv2 drv0) (list))))))))
 
-(unless (which (git-command)) (test-skip 1))
 (test-equal "channel-news, no news"
   '()
   (with-temporary-git-repository directory
@@ -318,7 +365,6 @@
             (latest  (reference-name->oid repository "HEAD")))
         (channel-news-for-commit channel (oid->string latest))))))
 
-(unless (which (git-command)) (test-skip 1))
 (test-assert "channel-news, one entry"
   (with-temporary-git-repository directory
       `((add ".guix-channel"
@@ -406,7 +452,6 @@
                          (channel-news-for-commit channel commit5 commit1))
                     '(#f "tag-for-first-news-entry")))))))
 
-(unless (which (git-command)) (test-skip 1))
 (test-assert "channel-news, annotated tag"
   (with-temporary-git-repository directory
       `((add ".guix-channel"
@@ -453,7 +498,6 @@
                          (channel-news-for-commit channel commit2))
                     (list commit1)))))))
 
-(unless (which (git-command)) (test-skip 1))
 (test-assert "latest-channel-instances, missing introduction for 'guix'"
   (with-temporary-git-repository directory
       '((add "a.txt" "A")
