@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2012-2022 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2012-2025 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -24,10 +24,19 @@
   #:use-module (guix utils)
   #:use-module ((gcrypt hash) #:prefix gcrypt:)
   #:use-module (guix base32)
+  #:use-module ((guix git) #:select (with-repository))
+  #:use-module (guix config)
   #:use-module (guix tests)
+  #:use-module (guix tests git)
   #:use-module (guix tests http)
   #:use-module ((guix packages) #:select (package-derivation base32))
-  #:use-module ((guix build utils) #:select (executable-file?))
+  #:use-module ((guix build utils)
+                #:select (executable-file? strip-store-file-name which))
+  #:use-module ((gnu build linux-container)
+                #:select (unprivileged-user-namespace-supported?))
+  #:use-module ((guix hash) #:select (file-hash*))
+  #:use-module ((git oid) #:select (oid->string))
+  #:use-module ((git reference) #:select (reference-name->oid))
   #:use-module (gnu packages bootstrap)
   #:use-module ((gnu packages guile) #:select (guile-1.8))
   #:use-module (srfi srfi-1)
@@ -49,6 +58,14 @@
 ;; Globally disable grafts because they can trigger early builds.
 (%graft? #f)
 
+;; This can happen when someone is running tests without --disable-chroot and
+;; with either slirp4netns or /dev/net/tun unavailable.
+(define builder-network-isolated?
+  (and (target-linux? %system)
+       (unprivileged-user-namespace-supported?)
+       (or (not (which "slirp4netns"))
+           (not (file-exists? "/dev/net/tun")))))
+
 (define (bootstrap-binary name)
   (let ((bin (search-bootstrap-binary name (%current-system))))
     (and %store
@@ -144,29 +161,28 @@
 (test-assert "derivation fails but keep going"
   ;; In keep-going mode, 'build-derivations' should fail because of D1, but it
   ;; must return only after D2 has succeeded.
-  (with-store store
-    (let* ((d1 (derivation %store "fails"
-                           %bash `("-c" "false")
-                           #:sources (list %bash)))
-           (d2 (build-expression->derivation %store "sleep-then-succeed"
-                                             `(begin
-                                                ,(random-text)
-                                                ;; XXX: Hopefully that's long
-                                                ;; enough that D1 has already
-                                                ;; failed.
-                                                (sleep 2)
-                                                (mkdir %output)))))
-      (set-build-options %store
-                         #:use-substitutes? #f
-                         #:keep-going? #t)
-      (guard (c ((store-protocol-error? c)
-                 (and (= 100 (store-protocol-error-status c))
-                      (string-contains (store-protocol-error-message c)
-                                       (derivation-file-name d1))
-                      (not (valid-path? %store (derivation->output-path d1)))
-                      (valid-path? %store (derivation->output-path d2)))))
-        (build-derivations %store (list d1 d2))
-        #f))))
+  (let* ((d1 (derivation %store "fails"
+                         %bash `("-c" "false")
+                         #:sources (list %bash)))
+         (d2 (build-expression->derivation %store "sleep-then-succeed"
+                                           `(begin
+                                              ,(random-text)
+                                              ;; XXX: Hopefully that's long
+                                              ;; enough that D1 has already
+                                              ;; failed.
+                                              (sleep 2)
+                                              (mkdir %output)))))
+    (set-build-options %store
+                       #:use-substitutes? #f
+                       #:keep-going? #t)
+    (guard (c ((store-protocol-error? c)
+               (and (= 100 (store-protocol-error-status c))
+                    (string-contains (store-protocol-error-message c)
+                                     (derivation-file-name d1))
+                    (not (valid-path? %store (derivation->output-path d1)))
+                    (valid-path? %store (derivation->output-path d2)))))
+      (build-derivations %store (list d1 d2))
+      #f)))
 
 (test-assert "identical files are deduplicated"
   ;; Note: DATA must be longer than %DEDUPLICATION-MINIMUM-SIZE.
@@ -195,7 +211,7 @@
                    (stat:ino (lstat file2))))))))
 
 (test-equal "built-in-builders"
-  '("download")
+  '("download" "git-download")
   (built-in-builders %store))
 
 (test-assert "unknown built-in builder"
@@ -256,6 +272,21 @@
       (build-derivations %store (list drv))
       #f)))
 
+(test-assert "'download' built-in builder, no fixed-output hash"
+  ;; 'guix perform-download' should bail out with a message saying "not a
+  ;; fixed-output derivation".
+  (with-http-server '((200 "This should not be downloaded."))
+    (let* ((drv (derivation %store "download-without-hash"
+                            "builtin:download" '()
+                            #:env-vars `(("url"
+                                          . ,(object->string (%local-url))))
+                            #:hash-algo 'sha256
+                            #:hash #f)))
+      (guard (c ((store-protocol-error? c)
+                 (string-contains (store-protocol-error-message c) "failed")))
+        (build-derivations %store (list drv))
+        #f))))
+
 (test-assert "'download' built-in builder, check mode"
   ;; Make sure rebuilding the 'builtin:download' derivation in check mode
   ;; works.  See <http://bugs.gnu.org/25089>.
@@ -275,6 +306,106 @@
                          get-string-all)
                        text))))))
 
+(define %nonce (random-text))
+
+(test-equal "'git-download' built-in builder"
+  `(("/a.txt" . ,%nonce)
+    ("/b.scm" . "#t"))
+  (let ((nonce (random-text)))
+    (with-served-temporary-git-repository directory port
+        `((add "a.txt" ,%nonce)
+          (add "b.scm" "#t")
+          (commit ,nonce))
+      (let* ((commit (with-repository directory repository
+                       (oid->string
+                        (reference-name->oid repository "HEAD"))))
+             (drv (derivation %store "git-download"
+                              "builtin:git-download" '()
+                              #:env-vars
+                              `(("url"
+                                 . ,(object->string
+                                     (string-append "git://localhost:"
+                                                    (number->string port)
+                                                    "/")))
+                                ("commit" . ,commit))
+                              #:hash-algo 'sha256
+                              #:hash (file-hash* directory
+                                                 #:algorithm
+                                                 (gcrypt:hash-algorithm
+                                                  gcrypt:sha256)
+                                                 #:recursive? #t)
+                              #:recursive? #t)))
+        (build-derivations %store (list drv))
+        (directory-contents (derivation->output-path drv) get-string-all)))))
+
+(test-assert "'git-download' built-in builder, invalid hash"
+  (with-served-temporary-git-repository directory port
+      `((add "a.txt" "AAA")
+        (add "b.scm" "#t")
+        (commit "Commit!"))
+    (let* ((commit (with-repository directory repository
+                     (oid->string
+                      (reference-name->oid repository "HEAD"))))
+           (drv (derivation %store "git-download"
+                            "builtin:git-download" '()
+                            #:env-vars
+                            `(("url"
+                               . ,(object->string
+                                   (string-append "git://localhost:"
+                                                  (number->string port)
+                                                  "/")))
+                              ("commit" . ,commit))
+                            #:hash-algo 'sha256
+                            #:hash (gcrypt:sha256 #vu8())
+                            #:recursive? #t)))
+      (guard (c ((store-protocol-error? c)
+                 (string-contains (store-protocol-error-message c) "failed")))
+        (build-derivations %store (list drv))
+        #f))))
+
+(test-assert "'git-download' built-in builder, invalid commit"
+  (with-served-temporary-git-repository directory port
+      `((add "a.txt" "AAA")
+        (add "b.scm" "#t")
+        (commit "Commit!"))
+    (let* ((drv (derivation %store "git-download"
+                            "builtin:git-download" '()
+                            #:env-vars
+                            `(("url"
+                               . ,(object->string
+                                   (string-append "git://localhost:"
+                                                  (number->string port)
+                                                  "/")))
+                              ("commit"
+                               . "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"))
+                            #:hash-algo 'sha256
+                            #:hash (gcrypt:sha256 #vu8())
+                            #:recursive? #t)))
+      (guard (c ((store-protocol-error? c)
+                 (string-contains (store-protocol-error-message c) "failed")))
+        (build-derivations %store (list drv))
+        #f))))
+
+(test-assert "'git-download' built-in builder, not found"
+  (with-served-temporary-git-repository directory port
+    '()
+    (let* ((drv (derivation %store "git-download"
+                            "builtin:git-download" '()
+                            #:env-vars
+                            `(("url" . ,(object->string
+                                         (string-append "git://localhost:"
+                                                        (number->string port)
+                                                        "/nonexistent")))
+                              ("commit"
+                               . "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"))
+                            #:hash-algo 'sha256
+                            #:hash (gcrypt:sha256 #vu8())
+                            #:recursive? #t)))
+      (guard (c ((store-protocol-error? c)
+                 (string-contains (store-protocol-error-message c) "failed")))
+        (build-derivations %store (list drv))
+        #f))))
+
 (test-equal "derivation-name"
   "foo-0.0"
   (let ((drv (derivation %store "foo-0.0" %bash '())))
@@ -316,6 +447,13 @@
                                  #:hash hash #:hash-algo 'sha256)))
     (fixed-output-derivation? drv)))
 
+(test-assert "fixed-output-derivation?, no hash"
+  ;; A derivation that has #:hash-algo and #:hash #f is *not* fixed-output.
+  (let* ((drv (derivation %store "not-quite-fixed"
+                          "builtin:download" '()
+                          #:hash #f #:hash-algo 'sha256)))
+    (not (fixed-output-derivation? drv))))
+
 (test-equal "fixed-output derivation"
   '(sha1 sha256 sha512)
   (map (lambda (hash-algorithm)
@@ -329,7 +467,7 @@
                                      (string-append
                                       "fixed-" (symbol->string hash-algorithm))
                                      %bash `(,builder)
-                                     #:sources `(,builder) ;optional
+                                     #:sources (list %bash builder)
                                      #:hash hash
                                      #:hash-algo hash-algorithm)))
            (build-derivations %store (list drv))
@@ -348,9 +486,11 @@
          (hash       (gcrypt:sha256 (string->utf8 "hello")))
          (drv1       (derivation %store "fixed"
                                  %bash `(,builder1)
+                                 #:sources (list %bash builder1)
                                  #:hash hash #:hash-algo 'sha256))
          (drv2       (derivation %store "fixed"
                                  %bash `(,builder2)
+                                 #:sources (list %bash builder2)
                                  #:hash hash #:hash-algo 'sha256))
          (succeeded? (build-derivations %store (list drv1 drv2))))
     (and succeeded?
@@ -363,7 +503,7 @@
          (hash       (gcrypt:sha256 (string->utf8 "hello")))
          (drv        (derivation %store "fixed-rec"
                                  %bash `(,builder)
-                                 #:sources (list builder)
+                                 #:sources (list %bash builder)
                                  #:hash (base32 "0sg9f58l1jj88w6pdrfdpj5x9b1zrwszk84j81zvby36q9whhhqa")
                                  #:hash-algo 'sha256
                                  #:recursive? #t))
@@ -374,6 +514,83 @@
                         (call-with-input-file p get-bytevector-all))
                 (bytevector? (query-path-hash %store p)))))))
 
+(test-assert "fixed-output derivation, invalid hash size"
+  (guard (c ((store-protocol-error? c)
+             (string-contains-ci (store-protocol-error-message c)
+                                 "invalid SHA512 hash")))
+    (derivation %store "download-with-invalid-hash"
+                "builtin:download" '()
+                #:env-vars `(("url"
+                              . ,(object->string "http://example.org")))
+                #:hash-algo 'sha512
+                #:hash #vu8(1 2 3))
+    #f))
+
+(unless (not builder-network-isolated?) (test-skip 1))
+(test-assert "fixed-output derivation, network access, localhost"
+  ;; Test a fixed-output derivation connecting to "localhost".
+  (let ((text (random-text)))
+    (with-http-server `((200 ,text))
+      (let* ((drv (build-expression->derivation
+                   %store
+                   "fixed-output-download"
+                   `(begin
+                      (use-modules (web client)
+                                   (srfi srfi-11)
+                                   (rnrs io ports))
+
+                      ;; Neutralize 'set-port-encoding!' because
+                      ;; guile-bootstrap cannot open iconv descriptors
+                      ;; contrary to what 'read-response' expects.
+                      (set! (@ (guile) set-port-encoding!) (const #t))
+
+                      (let-values (((response body)
+                                    (http-get ,(%local-url)
+                                              #:decode-body? #f)))
+                        (format #t "response: ~s~%body: ~s~%"
+                                response body)
+                        (call-with-output-file %output
+                          (lambda (port)
+                            (put-bytevector port body)))))
+                   #:hash-algo 'sha256
+                   #:hash (gcrypt:sha256 (string->utf8 text)))))
+        (and (build-derivations %store (list drv))
+             (string=? (call-with-input-file (derivation->output-path drv)
+                         get-string-all)
+                       text))))))
+
+(unless (and (network-reachable?) (not builder-network-isolated?))
+  (test-skip 1))
+(test-assert "fixed-output derivation, network access, external host"
+  ;; Test a fixed-output derivation connecting to an external server.
+  (let* ((drv (build-expression->derivation
+               %store
+               (string-append (number->string (random (expt 2 64) (%seed))
+                                              16)
+                              "-gplv3.txt")
+               '(begin
+                  (use-modules (web client)
+                               (srfi srfi-11)
+                               (rnrs io ports))
+
+                  ;; DNS working?
+                  (pk 'addr (getaddrinfo "www.gnu.org" "https"))
+
+                  ;; Neutralize 'set-port-encoding!' because
+                  ;; guile-bootstrap cannot open iconv descriptors
+                  ;; contrary to what 'read-response' expects.
+                  (set! (@ (guile) set-port-encoding!) (const #t))
+
+                  (let-values (((response body)
+                                (http-get "http://www.gnu.org/licenses/gpl-3.0.txt"
+                                          #:decode-body? #f)))
+                    (call-with-output-file %output
+                      (lambda (port)
+                        (put-bytevector port body)))))
+               #:hash-algo 'sha256
+               #:hash (base32 "11k9nggwk1mgsrkdwgdjz65avrradxlpdgrdkc7ryjgn8jbxqwir"))))
+    (build-derivations %store (list drv))))
+
 (test-assert "derivation with a fixed-output input"
   ;; A derivation D using a fixed-output derivation F doesn't has the same
   ;; output path when passed F or F', as long as F and F' have the same output
@@ -385,9 +602,11 @@
          (hash       (gcrypt:sha256 (string->utf8 "hello")))
          (fixed1     (derivation %store "fixed"
                                  %bash `(,builder1)
+                                 #:sources (list %bash builder1)
                                  #:hash hash #:hash-algo 'sha256))
          (fixed2     (derivation %store "fixed"
                                  %bash `(,builder2)
+                                 #:sources (list %bash builder2)
                                  #:hash hash #:hash-algo 'sha256))
          (fixed-out  (derivation->output-path fixed1))
          (builder3   (add-text-to-store
@@ -422,9 +641,11 @@
          (hash       (gcrypt:sha256 (string->utf8 "hello")))
          (fixed1     (derivation %store "fixed"
                                  %bash `(,builder1)
+                                 #:sources (list %bash builder1)
                                  #:hash hash #:hash-algo 'sha256))
          (fixed2     (derivation %store "fixed"
                                  %bash `(,builder2)
+                                 #:sources (list %bash builder2)
                                  #:hash hash #:hash-algo 'sha256))
          (builder3   (add-text-to-store %store "builder.sh"
                                         "echo fake builder"))
@@ -454,21 +675,21 @@
                                       '()))
          (hash     (gcrypt:sha256 (string->utf8 "hello")))
          (drv1     (derivation %store "fixed" %bash (list builder1)
-                               #:sources (list builder1)
+                               #:sources (list %bash builder1)
                                #:hash hash #:hash-algo 'sha256))
          (drv2     (derivation %store "fixed" %bash (list builder2)
-                               #:sources (list builder2)
+                               #:sources (list %bash builder2)
                                #:hash hash #:hash-algo 'sha256))
          (drv3a    (derivation %store "fixed-user" %bash (list builder3)
                                #:outputs '("one" "two")
-                               #:sources (list builder3)
+                               #:sources (list %bash builder3)
                                #:inputs (list (derivation-input drv1))))
          (drv3b    (derivation %store "fixed-user" %bash (list builder3)
                                #:outputs '("one" "two")
-                               #:sources (list builder3)
+                               #:sources (list %bash builder3)
                                #:inputs (list (derivation-input drv2))))
          (drv4     (derivation %store "fixed-user-user" %bash (list builder1)
-                               #:sources (list builder1)
+                               #:sources (list %bash builder1)
                                #:inputs (list (derivation-input drv3a '("one"))
                                               (derivation-input drv3b '("two"))))))
     (match (derivation-inputs drv4)
@@ -543,6 +764,22 @@
                     read-derivation)))
     (equal? drv* drv)))
 
+(test-assert "read-derivation with hash = #f"
+  ;; Passing #:hash-algo together with #:hash #f is accepted and #:hash-algo
+  ;; is preserved.  However it is not a fixed-output derivation.  It used to
+  ;; be that 'read-derivation' would incorrectly return #vu8() instead of #f
+  ;; for the hash in this case:
+  ;; <https://lists.gnu.org/archive/html/guix-devel/2023-01/msg00040.html>.
+  (let* ((drv1 (derivation %store "almost-fixed-output"
+                           "builtin:download" '()
+                           #:env-vars `(("url" . "http://example.org"))
+                           #:hash-algo 'sha256
+                           #:hash #f))
+         (drv2 (call-with-input-file (derivation-file-name drv1)
+                 read-derivation)))
+    (and (not (eq? drv1 drv2))             ;ensure memoization doesn't kick in
+         (equal? drv1 drv2))))
+
 (test-assert "multiple-output derivation, derivation-path->output-path"
   (let* ((builder    (add-text-to-store %store "builder.sh"
                                         "echo one > $out ; echo two > $second"
@@ -716,6 +953,20 @@
          (call-with-input-file (derivation->output-path drv)
            get-string-all))))
 
+(test-assert "builder is outside the store"
+  ;; Ensure that attempts to build derivations whose builder is outside the
+  ;; store are rejected.  This is a protection against attacks similar to
+  ;; CVE-2019-5736, which abuse the fact that /proc/self/exe can be opened
+  ;; even when it presents itself as a symlink to a file not in the chroot.
+  (let* ((builder (add-file-tree-to-store %store
+                                          `("builder" symlink "/proc/self/exe")))
+         (drv (derivation %store "attempt-to-run-guix-daemon" builder '()
+                          #:env-vars
+                          '(("LD_PRELOAD" . "attacker-controlled.so")))))
+    (guard (c ((store-protocol-error? c) c))
+      (build-derivations %store (list drv))
+      #f)))
+
 
 (define %coreutils
   (false-if-exception
@@ -736,7 +987,7 @@
                                     ,(string-append
                                       (derivation->output-path %coreutils)
                                       "/bin")))
-                      #:sources (list builder)
+                      #:sources (list %bash builder)
                       #:inputs (list (derivation-input %coreutils))))
          (succeeded?
           (build-derivations %store (list drv))))
@@ -1015,6 +1266,32 @@
                                          #:mode (build-mode check))
                   (list drv dep))))))
 
+(test-equal "derivation-build-plan, topological ordering"
+  (make-list 5 '("0.drv" "1.drv" "2.drv" "3.drv" "4.drv"))
+  (with-store store
+    (define (test _)
+      (let* ((simple-derivation
+              (lambda (name . deps)
+                (build-expression->derivation
+                 store name
+                 `(begin ,(random-text) (mkdir %output))
+                 #:inputs (map (lambda (n dep)
+                                 (list (number->string n) dep))
+                               (iota (length deps))
+                               deps))))
+             (drv0 (simple-derivation "0"))
+             (drv1 (simple-derivation "1" drv0))
+             (drv2 (simple-derivation "2" drv1))
+             (drv3 (simple-derivation "3" drv2 drv0))
+             (drv4 (simple-derivation "4" drv3 drv1)))
+        (map (compose strip-store-file-name derivation-file-name)
+             (derivation-build-plan store (list (derivation-input drv4))))))
+
+    ;; This is probabilistic: if the traversal is buggy, it may or may not
+    ;; produce the wrong ordering, depending on a variety of actors.  Thus,
+    ;; try multiple times.
+    (map test (iota 5))))
+
 (test-assert "derivation-input-fold"
   (let* ((builder (add-text-to-store %store "my-builder.sh"
                                      "echo hello, world > \"$out\"\n"
