@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2014-2022 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2014-2025 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2021-2022 Maxime Devos <maximedevos@telenet.be>
 ;;;
 ;;; This file is part of GNU Guix.
@@ -47,6 +47,13 @@
 (define %store
   (open-connection-for-tests))
 
+(define (bootstrap-guile-effective-version)
+  ;; TODO The package version of %bootstrap-guile is incorrect for
+  ;; riscv64-linux
+  (if (string=? "riscv64-linux" (%current-system))
+      "3.0"
+      (package-version %bootstrap-guile)))
+
 ;; Globally disable grafts because they can trigger early builds.
 (%graft? #f)
 
@@ -244,6 +251,12 @@
       (let ((file (local-file (string-copy "../base32.scm"))))
         (local-file-absolute-file-name file)))))
 
+(test-equal "local-file, non-literal source relative file name"
+  (current-filename)
+  (let ((file (local-file (assume-source-relative-file-name
+                           (string-append "gexp" ".scm")))))
+    (local-file-absolute-file-name file)))
+
 (test-assert "local-file, relative file name, within gexp"
   (let* ((file     (search-path %load-path "guix/base32.scm"))
          (interned (add-to-store %store "base32.scm" #f "sha256" file)))
@@ -285,6 +298,53 @@
                  (equal? (scandir (string-append dir "/tests"))
                          '("." ".." "gexp.scm"))))))
 
+(test-assert "local-file, capture file at the right time"
+  (call-with-temporary-directory
+   (lambda (directory)
+     (call-with-output-file (in-vicinity directory "the-unique-file.txt")
+       (lambda (port)
+         (display "Hi!" port)))
+
+     (let ((file (with-directory-excursion directory
+                   ;; If the argument to 'local-file' were resolved when
+                   ;; 'local-file-absolute-file-name' is called, we'd get the
+                   ;; wrong result.
+                   (local-file (in-vicinity (getcwd)
+                                            "the-unique-file.txt")))))
+       (string=? (local-file-absolute-file-name file)
+                 (in-vicinity directory "the-unique-file.txt"))))))
+
+(test-assert "local-file, load through symlink"
+  ;; See <https://issues.guix.gnu.org/72867>.
+  (call-with-temporary-directory
+   (lambda (tmp-dir)
+     (with-directory-excursion tmp-dir
+       ;; create content file
+       (call-with-output-file "content"
+         (lambda (port) (display "Hi!" port)))
+       ;; Create a module that calls 'local-file' with the "content" file and
+       ;; returns its absolute file name.  An error is raised if the "content"
+       ;; file can't be found.
+       (call-with-output-file "test-local-file.scm"
+         (lambda (port) (display "\
+(define-module (test-local-file)
+  #:use-module (guix gexp))
+(define file (local-file \"content\" \"test-file\"))
+(local-file-absolute-file-name file)" port)))
+       (mkdir "dir")
+       (symlink "../test-local-file.scm" "dir/test-local-file.scm")
+       ;; 'local-file' in turn calls 'current-source-directory' which has an
+       ;; 'if' branching condition depending on whether 'file-name' is
+       ;; absolute or relative file name.  To test both of these branches we
+       ;; execute 'test-local-file.scm' symlink first as a module (corresponds
+       ;; to relative file name):
+       (dynamic-wind
+         (lambda () (set! %load-path (cons "dir" %load-path)))
+         (lambda () (resolve-module '(test-local-file) #:ensure #f))
+         (lambda () (set! %load-path (cdr %load-path))))
+       ;; and then as a regular code (corresponds to absolute file name):
+       (load (string-append tmp-dir "/dir/test-local-file.scm"))))))
+
 (test-assert "one plain file"
   (let* ((file     (plain-file "hi" "Hello, world!"))
          (exp      (gexp (display (ungexp file))))
@@ -393,6 +453,30 @@
                  (list item))
          (null? (lowered-gexp-inputs lexp)))))
 
+(test-equal "gexp references non-existent output"
+  "no-default-output"
+  (guard (c ((derivation-missing-output-error? c)
+             (derivation-name (derivation-error-derivation c))))
+    (let* ((obj  (computed-file "no-default-output"
+                                #~(mkdir #$output:bar)))
+           (exp  #~(symlink #$obj #$output))
+           (drv  (run-with-store %store (lower-gexp exp))))
+      (pk 'oops! drv #f))))
+
+(test-assert "gexp-input, as first-class input"
+  ;; Insert a <gexp-input> record in a gexp as a way to specify which output
+  ;; of OBJ should be used.
+  (let* ((obj  (computed-file "foo" #~(mkdir #$output:bar)))
+         (exp  #~(list #$(gexp-input obj "bar")))
+         (drv  (run-with-store %store (lower-object obj)))
+         (item (derivation->output-path drv "bar"))
+         (lexp (run-with-store %store (lower-gexp exp))))
+    (and (match (lowered-gexp-inputs lexp)
+           ((input)
+            (eq? (derivation-input-derivation input) drv)))
+         (equal? (lowered-gexp-sexp lexp)
+                 `(list ,item)))))
+
 (test-assertm "with-parameters for %current-system"
   (mlet* %store-monad ((system -> (match (%current-system)
                                     ("aarch64-linux" "x86_64-linux")
@@ -414,6 +498,26 @@
     (return (string=? (derivation-file-name drv)
                       (derivation-file-name result)))))
 
+(test-assertm "with-parameters for %graft?"
+  (mlet* %store-monad ((replacement -> (package
+                                         (inherit %bootstrap-guile)
+                                         (name (string-upcase
+                                                (package-name
+                                                 %bootstrap-guile)))))
+                       (guile -> (package
+                                   (inherit %bootstrap-guile)
+                                   (replacement replacement)))
+                       (drv0   (package->derivation %bootstrap-guile))
+                       (drv1   (package->derivation replacement))
+                       (obj0 -> (with-parameters ((%graft? #f))
+                                  guile))
+                       (obj1 -> (with-parameters ((%graft? #t))
+                                  guile))
+                       (result0 (lower-object obj0))
+                       (result1 (lower-object obj1)))
+    (return (and (eq? drv0 result0)
+                 (eq? drv1 result1)))))
+
 (test-assert "with-parameters + file-append"
   (let* ((system (match (%current-system)
                    ("aarch64-linux" "x86_64-linux")
@@ -430,6 +534,15 @@
        (string=? result
                  (string-append (derivation->output-path drv)
                                 "/bin/touch"))))))
+
+(test-assert "with-parameters + store item"
+  (let* ((file (add-text-to-store %store "hello.txt" "Hello, world!"))
+         (obj (with-parameters ((%current-system "aarch64-linux"))
+                file))
+         (lowered (run-with-store %store
+                    (lower-object obj))))
+    (string=? lowered file)))
+
 (test-equal "let-system"
   (list `(begin ,(%current-system) #t) '(system-binding)
         'low '() '())
@@ -637,7 +750,8 @@
   (mlet* %store-monad ((exp -> (gexp (display (ungexp %bootstrap-guile))))
                        (guile  (package-file %bootstrap-guile))
                        (sexp   (gexp->sexp exp (%current-system) #f))
-                       (drv    (gexp->file "foo" exp))
+                       (drv    (gexp->file "foo" exp
+                                           #:guile %bootstrap-guile))
                        (out -> (derivation->output-path drv))
                        (done   (built-derivations (list drv)))
                        (refs   (references* out)))
@@ -648,7 +762,8 @@
   (mlet* %store-monad ((exp -> #~#$(file-append %bootstrap-guile
                                                 "/bin/guile"))
                        (guile  (package-file %bootstrap-guile))
-                       (drv    (gexp->file "foo" exp))
+                       (drv    (gexp->file "foo" exp
+                                           #:guile %bootstrap-guile))
                        (out -> (derivation->output-path drv))
                        (done   (built-derivations (list drv)))
                        (refs   (references* out)))
@@ -661,7 +776,9 @@
                                 #~(define foo 'bar)
                                 #~(define guile #$%bootstrap-guile)))
                        (guile  (package-file %bootstrap-guile))
-                       (drv    (gexp->file "splice" exp #:splice? #t))
+                       (drv    (gexp->file "splice" exp
+                                           #:splice? #t
+                                           #:guile %bootstrap-guile))
                        (out -> (derivation->output-path drv))
                        (done   (built-derivations (list drv)))
                        (refs   (references* out)))
@@ -826,38 +943,39 @@
                          (call-with-output-file (string-append #$output "/two")
                            (lambda (port)
                              (display "This is the second one." port))))))
-        (build-drv #~(begin
-                       (use-modules (guix build store-copy)
-                                    (guix build utils)
-                                    (srfi srfi-1))
+        (build-drv
+         (with-imported-modules '((guix build store-copy)
+                                  (guix build syscalls)
+                                  (guix progress)
+                                  (guix records)
+                                  (guix sets)
+                                  (guix build utils))
+           #~(begin
+               (use-modules (guix build store-copy)
+                            (guix build utils)
+                            (srfi srfi-1))
 
-                       (define (canonical-file? file)
-                         ;; Copied from (guix tests).
-                         (let ((st (lstat file)))
-                           (or (not (string-prefix? (%store-directory) file))
-                               (eq? 'symlink (stat:type st))
-                               (and (= 1 (stat:mtime st))
-                                    (zero? (logand #o222 (stat:mode st)))))))
+               (define (canonical-file? file)
+                 ;; Copied from (guix tests).
+                 (let ((st (lstat file)))
+                   (or (not (string-prefix? (%store-directory) file))
+                       (eq? 'symlink (stat:type st))
+                       (and (= 1 (stat:mtime st))
+                            (zero? (logand #o222 (stat:mode st)))))))
 
-                       (mkdir #$output)
-                       (populate-store '("graph") #$output
-                                       #:deduplicate? #f)
+               (mkdir #$output)
+               (populate-store '("graph") #$output
+                               #:deduplicate? #f)
 
-                       ;; Check whether 'populate-store' canonicalizes
-                       ;; permissions and timestamps.
-                       (unless (every canonical-file? (find-files #$output))
-                         (error "not canonical!" #$output)))))
+               ;; Check whether 'populate-store' canonicalizes
+               ;; permissions and timestamps.
+               (unless (every canonical-file? (find-files #$output))
+                 (error "not canonical!" #$output))))))
     (mlet* %store-monad ((one (gexp->derivation "one" build-one))
                          (two (gexp->derivation "two" (build-two one)))
                          (drv (gexp->derivation "store-copy" build-drv
                                                 #:references-graphs
-                                                `(("graph" ,two))
-                                                #:modules
-                                                '((guix build store-copy)
-                                                  (guix progress)
-                                                  (guix records)
-                                                  (guix sets)
-                                                  (guix build utils))))
+                                                `(("graph" ,two))))
                          (ok? (built-derivations (list drv)))
                          (out -> (derivation->output-path drv)))
       (let ((one (derivation->output-path one))
@@ -902,6 +1020,33 @@
          (and (file=? (string-append dir "/a/b/c") q-scm* stat)
               (file=? (string-append dir "/p/q") plain* stat)))))))
 
+(test-assert "imported-files does not create symlinks"
+  ;; 'imported-files' should always produce a directory with regular files,
+  ;; whether or not it's going through 'imported-files/derivation'.
+  ;; See <https://issues.guix.gnu.org/73275>.
+  (call-with-temporary-directory
+   (lambda (directory)
+     (symlink (search-path %load-path "guix/store.scm")
+              (in-vicinity directory "store.scm"))
+
+     (run-with-store %store
+       (mlet* %store-monad
+           ((files1 -> `(("x" . ,(in-vicinity directory "store.scm"))))
+            (files2 -> `(,@files1
+                         ("y" . ,(plain-file "foo.scm" "#t"))))
+            (import1 (imported-files files1))
+            (import2-drv (imported-files files2))
+            (import2 -> (derivation->output-path import2-drv))
+            (_ (built-derivations (list import2-drv))))
+         (return (and (eq? (stat:type (lstat (in-vicinity import1 "x")))
+                           'regular)
+                      (eq? (stat:type (lstat (in-vicinity import2 "x")))
+                           'regular)
+                      (file=? (in-vicinity import1 "x")
+                              (search-path %load-path "guix/store.scm"))
+                      (file=? (in-vicinity import2 "x")
+                              (search-path %load-path "guix/store.scm")))))))))
+
 (test-equal "gexp-modules & ungexp"
   '((bar) (foo))
   ((@@ (guix gexp) gexp-modules)
@@ -918,7 +1063,8 @@
   (let ((make-file (lambda ()
                      ;; Use 'eval' to make sure we get an object that's not
                      ;; 'eq?' nor 'equal?' due to the closures it embeds.
-                     (eval '(scheme-file "bar.scm" #~(define-module (bar)))
+                     (eval '(scheme-file "bar.scm" #~(define-module (bar))
+                                         #:guile %bootstrap-guile)
                            (current-module)))))
     (define result
       ((@@ (guix gexp) gexp-modules)
@@ -1010,7 +1156,8 @@ importing.* \\(guix config\\) from the host"
                                          #:export (the-answer))
 
                                        (define the-answer 42))
-                               #:splice? #t))
+                               #:splice? #t
+                               #:guile %bootstrap-guile))
        (build -> (with-imported-modules `(((foo bar) => ,module)
                                           (guix build utils))
                    #~(begin
@@ -1055,7 +1202,8 @@ importing.* \\(guix config\\) from the host"
 
                                       (define (multiply x)
                                         (* the-answer x)))
-                               #:splice? #t))
+                               #:splice? #t
+                               #:guile %bootstrap-guile))
        (build -> (with-extensions (list extension)
                    (with-imported-modules `((guix build utils)
                                             ((foo) => ,module))
@@ -1067,8 +1215,8 @@ importing.* \\(guix config\\) from the host"
                              (write (list the-answer (multiply 2))
                                     port)))))))
        (drv      (gexp->derivation "thingie" build
-                                   ;; %BOOTSTRAP-GUILE is 2.0.
-                                   #:effective-version "2.0"))
+                                   #:effective-version
+                                   (bootstrap-guile-effective-version)))
        (out ->   (derivation->output-path drv)))
     (mbegin %store-monad
       (built-derivations (list drv))
@@ -1088,7 +1236,8 @@ importing.* \\(guix config\\) from the host"
                          mkdir-p
                          the-answer))))
        (lexp     (lower-gexp exp
-                             #:effective-version "2.0")))
+                             #:effective-version
+                             (bootstrap-guile-effective-version))))
     (define (matching-input drv output)
       (lambda (input)
         (and (eq? (derivation-input-derivation input) drv)
@@ -1102,12 +1251,15 @@ importing.* \\(guix config\\) from the host"
                          (lowered-gexp-inputs lexp))
                    (member (string-append
                             (derivation->output-path extension-drv)
-                            "/share/guile/site/2.0")
+                            "/share/guile/site/"
+                            (bootstrap-guile-effective-version))
                            (lowered-gexp-load-path lexp))
                    (= 2 (length (lowered-gexp-load-path lexp)))
                    (member (string-append
                             (derivation->output-path extension-drv)
-                            "/lib/guile/2.0/site-ccache")
+                            "/lib/guile/"
+                            (bootstrap-guile-effective-version)
+                            "/site-ccache")
                            (lowered-gexp-load-compiled-path lexp))
                    (= 2 (length (lowered-gexp-load-compiled-path lexp)))
                    (eq? (derivation-input-derivation (lowered-gexp-guile lexp))
@@ -1117,7 +1269,10 @@ importing.* \\(guix config\\) from the host"
   (mlet* %store-monad ((thing -> (program-file "prog" #~(display "hi!")))
                        (exp -> #~(list #$(raw-derivation-file thing)))
                        (drv  (lower-object thing))
-                       (lexp (lower-gexp exp #:effective-version "2.0")))
+                       (lexp (lower-gexp
+                              exp
+                              #:effective-version
+                              (bootstrap-guile-effective-version))))
     (return (and (equal? `(list ,(derivation-file-name drv))
                          (lowered-gexp-sexp lexp))
                  (equal? (list (derivation-file-name drv))
@@ -1407,7 +1562,8 @@ importing.* \\(guix config\\) from the host"
 
 (test-assertm "scheme-file"
   (let* ((text   (plain-file "foo" "Hello, world!"))
-         (scheme (scheme-file "bar" #~(list "foo" #$text))))
+         (scheme (scheme-file "bar" #~(list "foo" #$text)
+                              #:guile %bootstrap-guile)))
     (mlet* %store-monad ((drv  (lower-object scheme))
                          (text (lower-object text))
                          (out -> (derivation->output-path drv)))
@@ -1627,6 +1783,36 @@ importing.* \\(guix config\\) from the host"
                            read)
                          refs)))))))
 
+(test-assertm "references-file, non-default output"
+  (let* ((exp      #~(begin
+                       (mkdir #$output)
+                       (symlink #$%bootstrap-guile #$output:extra)))
+         (computed (computed-file "computed" exp
+                                  #:guile %bootstrap-guile))
+         (refs1    (references-file computed
+                                    #:guile %bootstrap-guile))
+         ;; Wrap COMPUTE in 'gexp-input' to get the "extra" output.
+         (refs2    (references-file (gexp-input computed "extra")
+                                    #:guile %bootstrap-guile)))
+    (mlet* %store-monad ((drv0 (lower-object %bootstrap-guile))
+                         (drv1 (lower-object computed))
+                         (drv2 (lower-object refs2))
+                         (drv3 (lower-object refs1)))
+      (mbegin %store-monad
+        (built-derivations (list drv2 drv3))
+        (mlet %store-monad ((refs ((store-lift requisites)
+                                   (list (derivation->output-path
+                                          drv1 "extra")))))
+          (return
+           (and (lset= string=?
+                       (call-with-input-file (derivation->output-path drv2)
+                         read)
+                       refs)
+                (lset= string=?
+                       (call-with-input-file (derivation->output-path drv3)
+                         read)
+                       (list (derivation->output-path drv1))))))))))
+
 (test-assert "lower-object & gexp-input-error?"
   (guard (c ((gexp-input-error? c)
              (gexp-error-invalid-input c)))
@@ -1664,7 +1850,9 @@ importing.* \\(guix config\\) from the host"
 (test-assertm "gexp->file, cross-compilation"
   (mlet* %store-monad ((target -> "aarch64-linux-gnu")
                        (exp    -> (gexp (list (ungexp coreutils))))
-                       (xdrv      (gexp->file "foo" exp #:target target))
+                       (xdrv      (gexp->file "foo" exp
+                                              #:target target
+                                              #:guile %bootstrap-guile))
                        (refs      (references*
                                    (derivation-file-name xdrv)))
                        (xcu       (package->cross-derivation coreutils
@@ -1677,7 +1865,8 @@ importing.* \\(guix config\\) from the host"
   (mlet* %store-monad ((target -> "aarch64-linux-gnu")
                        (_         (set-current-target target))
                        (exp    -> (gexp (list (ungexp coreutils))))
-                       (xdrv      (gexp->file "foo" exp))
+                       (xdrv      (gexp->file "foo" exp
+                                              #:guile %bootstrap-guile))
                        (refs      (references*
                                    (derivation-file-name xdrv)))
                        (xcu       (package->cross-derivation coreutils
