@@ -1,7 +1,8 @@
 # GNU Guix --- Functional package management for GNU
-# Copyright © 2012-2014, 2016-2022 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2012-2014, 2016-2025 Ludovic Courtès <ludo@gnu.org>
 # Copyright © 2020 Marius Bakke <mbakke@fastmail.com>
 # Copyright © 2021 Chris Marusich <cmmarusich@gmail.com>
+# Copyright © 2025 Maxim Cournoyer <maxim@guixotic.coop>
 #
 # This file is part of GNU Guix.
 #
@@ -25,7 +26,7 @@
 guix build --version
 
 # Should fail.
-! guix build -e +
+guix build -e + && false
 
 # Source-less packages are accepted; they just return nothing.
 guix build -e '(@ (gnu packages bootstrap) %bootstrap-glibc)' -S
@@ -92,7 +93,7 @@ cat > "$module_dir/foo.scm" <<EOF
 (use-modules (guix))
 ) ;extra closing paren
 EOF
-! guix build -f "$module_dir/foo.scm" 2> "$module_dir/stderr"
+guix build -f "$module_dir/foo.scm" 2> "$module_dir/stderr" && false
 grep "read error" "$module_dir/stderr"
 rm "$module_dir/stderr" "$module_dir/foo.scm"
 
@@ -161,6 +162,10 @@ export GUIX_PACKAGE_PATH
 guix build -d -S foo
 guix build -d -S foo | grep -e 'foo\.tar\.gz'
 
+# Make sure '-s' has an effect together with '-S'.
+test "$(guix build -Sd coreutils -s x86_64-linux)" \
+     != "$(guix build -Sd coreutils -s aarch64-linux)"
+
 # 'baz' has a replacement so we should be getting the replacement's source.
 (unset GUIX_BUILD_OPTIONS;
  test "`guix build -d -S baz`" = "`guix build -d -S foo`")
@@ -186,6 +191,18 @@ test `guix build -d --sources=transitive foo \
       | grep -e 'foo\.tar\.gz' -e 'bar\.tar\.gz' -e 'bar\.dat' \
       | wc -l` -eq 3
 
+# Building the inputs.
+guix build -D hello -n
+test `guix build -D hello -d \
+      | grep -e 'glibc.*\.drv$' -e 'gcc.*\.drv$' -e 'binutils.*\.drv$' \
+      | wc -l` -ge 3
+
+# Building the dependents.
+test `guix build -P1 libgcrypt -P1 libssh -d \
+      | grep -e 'guile-gcrypt.*\.drv$' -e 'guile-ssh.*\.drv$' \
+             -e 'libgcrypt-[0-9].*\.drv$' -e 'libssh-[0-9].*\.drv$' \
+      | sort -u \
+      | wc -l` -eq 4
 
 # Unbound variable in thunked field.
 cat > "$module_dir/foo.scm" <<EOF
@@ -199,7 +216,7 @@ cat > "$module_dir/foo.scm" <<EOF
     (inputs (quasiquote (("sed" ,sed))))))  ;unbound variable
 EOF
 
-! guix build package-with-something-wrong -n
+guix build package-with-something-wrong -n && false
 guix build package-with-something-wrong -n 2> "$module_dir/err" || true
 grep "unbound" "$module_dir/err"		     # actual error
 grep "forget.*(gnu packages base)" "$module_dir/err" # hint
@@ -240,7 +257,7 @@ cat > "$module_dir/cc-user.scm" <<EOF
 (define-module (cc-user))
 (make-thing 42)
 EOF
-! guix build -f "$module_dir/cc-user.scm" -n 2> "$module_dir/err"
+guix build -f "$module_dir/cc-user.scm" -n 2> "$module_dir/err" && false
 cat "$module_dir/err"
 grep "make-thing.*unbound" "$module_dir/err"		 # actual error
 grep "forget.*(bb-public)" "$module_dir/err"		 # hint
@@ -270,7 +287,7 @@ test "`guix build --log-file guile-bootstrap`" = "$log"
 test "`guix build --log-file $out`" = "$log"
 
 # Should fail because the name/version combination could not be found.
-! guix build hello-0.0.1 -n
+guix build hello-0.0.1 -n && false
 
 # Keep a symlink to the result, registered as a root.
 result="t-result-$$"
@@ -279,7 +296,7 @@ guix build -r "$result"					\
 test -x "$result/bin/guile"
 
 # Should fail, because $result already exists.
-! guix build -r "$result" -e '(@@ (gnu packages bootstrap) %bootstrap-guile)'
+guix build -r "$result" -e '(@@ (gnu packages bootstrap) %bootstrap-guile)' && false
 
 rm -f "$result"
 
@@ -318,12 +335,12 @@ drv2=`guix build inkscape -d --no-grafts --with-graft=glib=glib-networking`
 test "$drv1" = "$drv2"
 
 # Rewriting implicit inputs.
-drv1=`guix build hello -d`
-drv2=`guix build hello -d --with-input=gcc=gcc-toolchain`
+drv1=`guix build grep -d`
+drv2=`guix build grep -d --with-input=coreutils=hello`
 test "$drv1" != "$drv2"
-guix gc -R "$drv2" | grep `guix build -d gcc-toolchain`
+guix gc -R "$drv2" | grep `guix build -d hello`
 
-! guix build guile --with-input=libunistring=something-really-silly
+guix build guile --with-input=libunistring=something-really-silly && false
 
 # Deprecated/superseded packages.
 test "`guix build superseded -d`" = "`guix build bar -d`"
@@ -331,8 +348,8 @@ test "`guix build superseded -d`" = "`guix build bar -d`"
 # Parsing package names and versions.
 guix build -n time		# PASS
 guix build -n time@1.9		# PASS, version found
-! guix build -n time@3.2	# FAIL, version not found
-! guix build -n something-that-will-never-exist # FAIL
+guix build -n time@3.2 && false	# FAIL, version not found
+guix build -n something-that-will-never-exist && false	# FAIL
 
 # Invoking a monadic procedure.
 guix build -e "(begin
@@ -397,6 +414,29 @@ guix build -d -m "$module_dir/manifest.scm" \
 
 rm "$module_dir"/*.scm
 
+if [ -n "$BASH_VERSION" ]
+then
+    # Check whether we can load from a /dev/fd/N denoting a pipe, using this
+    # handy Bash-specific construct.
+    guix build -m <(echo '(specifications->manifest (list "guile"))') -n
+fi
+
+# Build a scheme-file object via multiple expressions, and validate it
+# produces the correct result when evaluated.
+scheme_file=$(guix build -e \
+     "(use-modules (guix gexp)) \
+      (scheme-file \"mathematics\" \
+       '(begin \
+         (define add +) \
+         (define multiply *) \
+         (add 5 (multiply 2 10)))
+       #:guile (@@ (gnu packages bootstrap) %bootstrap-guile))")
+guile -c \
+     "(begin \
+       (use-modules (guix ui) (rnrs base) (srfi srfi-26)) \
+       (assert (= 25 (call-with-input-file \"$scheme_file\" \
+                      (cut read/eval <>)))))"
+
 # Using 'GUIX_BUILD_OPTIONS'.
 GUIX_BUILD_OPTIONS="--dry-run --no-grafts"
 export GUIX_BUILD_OPTIONS
@@ -404,4 +444,6 @@ export GUIX_BUILD_OPTIONS
 guix build emacs
 
 GUIX_BUILD_OPTIONS="--something-completely-crazy"
-! guix build emacs
+guix build emacs && false
+
+exit 0
