@@ -1,5 +1,5 @@
 # GNU Guix --- Functional package management for GNU
-# Copyright © 2018, 2019, 2020 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2018-2020, 2023, 2025 Ludovic Courtès <ludo@gnu.org>
 # Copyright © 2020 Eric Bavier <bavier@posteo.net>
 #
 # This file is part of GNU Guix.
@@ -32,7 +32,8 @@ storedir="`guile -c '(use-modules (guix config))(display %storedir)'`"
 localstatedir="`guile -c '(use-modules (guix config))(display %localstatedir)'`"
 NIX_STORE_DIR="$storedir"
 GUIX_DAEMON_SOCKET="$localstatedir/guix/daemon-socket/socket"
-export NIX_STORE_DIR GUIX_DAEMON_SOCKET
+GUIX_BUILD_OPTIONS="--timeout=`guile -c '(use-modules (guix tests))(display %tests-build-timeout)'`"
+export NIX_STORE_DIR GUIX_DAEMON_SOCKET GUIX_BUILD_OPTIONS
 
 if ! guile -c '(use-modules (guix)) (exit (false-if-exception (open-connection)))'
 then
@@ -82,6 +83,7 @@ then
     tarball="`guix pack -R -S /Bin=bin sed`"
     (cd "$test_directory"; tar xvf "$tarball")
 
+    chmod +w "$test_directory"
     run_without_store "$test_directory/Bin/sed" --version > "$test_directory/output"
     grep 'GNU sed' "$test_directory/output"
 
@@ -91,7 +93,7 @@ then
     grep 'GNU sed' "$test_directory/output"
 
     # Check whether the exit code is preserved.
-    ! run_without_store "$test_directory/Bin/sed" --does-not-exist
+    run_without_store "$test_directory/Bin/sed" --does-not-exist && false
 
     chmod -Rf +w "$test_directory"; rm -rf "$test_directory"/*
 else
@@ -104,6 +106,7 @@ case "`uname -m`" in
 	tarball="`guix pack -RR -S /Bin=bin sed`"
 	tar tvf "$tarball" | grep /bin/proot
 	(cd "$test_directory"; tar xf "$tarball")
+	chmod +w "$test_directory"
 	run_without_store GUIX_EXECUTION_ENGINE="proot" \
 	"$test_directory/Bin/sed" --version > "$test_directory/output"
 	grep 'GNU sed' "$test_directory/output"
@@ -195,6 +198,7 @@ EOF
     # Run '/bin/daemon', which forks, then wait for the child, send it SIGHUP
     # so that it dumps its view of the store, and make sure the child and
     # parent both see the same store contents.
+    chmod +w "$test_directory"
     (cd "$test_directory"; run_without_store ./bin/daemon)
     wait_for_file "$test_directory/pid"
     kill -HUP $(cat "$test_directory/pid")
@@ -213,7 +217,12 @@ chmod -Rf +w "$test_directory"; rm -rf "$test_directory"/*
 # Ensure '-R' applies to propagated inputs.  Failing to do that, it would fail
 # with a profile collision error in this case because 'python-scipy'
 # propagates 'python-numpy'.  See <https://bugs.gnu.org/42510>.
-guix pack -RR python-numpy python-scipy --no-grafts -n
+#
+# 'python-scipy' depends on the package defined with the Guile variable
+# 'python-numpy' but the package specification 'python-numpy' doesn't always
+# refer to the same package (e.g. when multiple versions are defined) so the
+# Guile variable is directly referred to.
+guix pack -RR -e '(@ (gnu packages python-xyz) python-numpy)' python-scipy --no-grafts -n
 
 # Check that packages that mix executable and support files (e.g. git) in the
 # "binary" directories still work after wrapped.
@@ -241,6 +250,7 @@ cat >"$test_directory/manifest.scm" <<'EOF'
 EOF
 tarball="`guix pack -RR -S /opt= -m $test_directory/manifest.scm`"
 (cd "$test_directory"; tar xvf "$tarball")
+chmod +w "$test_directory"
 ( export GUIX_PROFILE=$test_directory/opt
   . $GUIX_PROFILE/etc/profile
   run_without_store "$test_directory/opt/bin/hello" > "$test_directory/output" )
