@@ -1,5 +1,5 @@
 # GNU Guix --- Functional package management for GNU
-# Copyright © 2014-2022 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2014-2022, 2024 Ludovic Courtès <ludo@gnu.org>
 # Copyright © 2017 Tobias Geerinckx-Rice <me@tobias.gr>
 # Copyright © 2018 Chris Marusich <cmmarusich@gmail.com>
 #
@@ -318,7 +318,7 @@ cat > "$tmpdir/config.scm" <<EOF
 
 (bad-local-file "whatever.scm")
 EOF
-! guix system build "$tmpdir/config.scm" -n
+guix system build "$tmpdir/config.scm" -n && false
 guix system build "$tmpdir/config.scm" -n 2>&1 | \
     grep "config\.scm:4:2: warning:.*whatever.*relative to current directory"
 
@@ -342,25 +342,39 @@ for example in gnu/system/examples/*.tmpl; do
 	    # 'asus-c201.tmpl' uses 'linux-libre-arm-generic', which is an
 	    # ARM-only package.
             options="--system=armhf-linux";;
+        *raspberry*)
+	    # The Raspberry Pi templates 'linux-libre-arm64-generic', which is
+	    # an ARM-only package.
+            options="--system=aarch64-linux";;
+        *plasma*)
+            # Some architectures do not support all the packages Plasma
+            # depends on so restrict to x86_64-linux.
+            options="--system=x86_64-linux";;
 	*vm-image*)
 	    # The VM image tries to build 'current-guix' as per 'guix pull'.
 	    # Skip it.
 	    continue
 	    ;;
+	*desktop*)
+	    # This image uses 'grub-efi-bootloader' so it needs a GPT
+	    # partition.
+	    options="-t efi-raw --system=x86_64-linux";;
 	*)
 	    options=""
 	    ;;
     esac
-    guix system -n disk-image $options "$example"
+    guix system -n image $options "$example"
 done
 
 # Make sure the desktop image can be built on major architectures.
-for system in x86_64-linux i686-linux aarch64-linux
+for system in x86_64-linux aarch64-linux
 do
-    guix system -n image -s "$system" gnu/system/examples/desktop.tmpl
+    guix system -n image -s "$system" -t efi-raw \
+	 gnu/system/examples/desktop.tmpl
 done
 
 # Verify that the images can be built.
+guix system -n container gnu/system/examples/bare-bones.tmpl
 guix system -n vm gnu/system/examples/bare-bones.tmpl
 guix system -n image gnu/system/images/pinebook-pro.scm
 guix system -n image -t qcow2 gnu/system/examples/bare-bones.tmpl
