@@ -1,16 +1,17 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2012, 2013 Cyril Roelandt <tipecaml@gmail.com>
 ;;; Copyright © 2014, 2015, 2016 Eric Bavier <bavier@member.fsf.org>
-;;; Copyright © 2014-2022 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2014-2025 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2015, 2016 Mathieu Lirzin <mthl@gnu.org>
 ;;; Copyright © 2016 Hartmut Goebel <h.goebel@crazy-compilers.com>
 ;;; Copyright © 2017 Alex Kost <alezost@gmail.com>
-;;; Copyright © 2017, 2022 Efraim Flashner <efraim@flashner.co.il>
+;;; Copyright © 2017, 2022, 2025 Efraim Flashner <efraim@flashner.co.il>
 ;;; Copyright © 2018, 2019 Arun Isaac <arunisaac@systemreboot.net>
 ;;; Copyright © 2020 Timothy Sample <samplet@ngyro.com>
 ;;; Copyright © 2020 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;; Copyright © 2021 Xinglu Chen <public@yoctocell.xyz>
-;;; Copyright © 2021 Maxime Devos <maximedevos@telenet.be>
+;;; Copyright © 2021, 2023 Maxime Devos <maximedevos@telenet.be>
+;;; Copyright © 2024 Gabriel Wicki <gabriel@erlikon.ch>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -43,7 +44,8 @@
   #:use-module (guix lint)
   #:use-module (guix ui)
   #:use-module (guix swh)
-  #:use-module ((guix gexp) #:select (gexp local-file gexp?))
+  #:use-module ((guix gexp)
+                #:select (gexp local-file computed-file gexp?))
   #:use-module ((guix utils) #:select (call-with-temporary-directory))
   #:use-module ((guix import hackage) #:select (%hackage-url))
   #:use-module ((guix import stackage) #:select (%stackage-url))
@@ -106,11 +108,18 @@
    (check-description-style
     (dummy-package "x" (description "")))))
 
+(test-equal "description: does not end with a period"
+  "description should end with a period"
+  (single-lint-warning-message
+   (let ((pkg (dummy-package "x"
+                             (description "This package provides a bad description"))))
+     (check-description-style pkg))))
+
 (test-equal "description: invalid Texinfo markup"
   "Texinfo markup in description is invalid"
   (single-lint-warning-message
    (check-description-style
-    (dummy-package "x" (description (identity "f{oo}b@r"))))))
+    (dummy-package "x" (description (identity "f{oo}b@r."))))))
 
 (test-equal "description: does not start with an upper-case letter"
   "description should start with an upper-case letter or digit"
@@ -119,6 +128,11 @@
                              (description "bad description."))))
      (check-description-style pkg))))
 
+(test-equal "description: may start with texinfo markup"
+  '()
+  (check-description-style
+   (dummy-package "x" (description "@emph{Maxwell Equations of Software}."))))
+
 (test-equal "description: may start with a digit"
   '()
   (let ((pkg (dummy-package "x"
@@ -131,6 +145,30 @@
                             (description "x is a dummy package."))))
     (check-description-style pkg)))
 
+(test-equal "description: may start with beginning of package name"
+  '()
+  (let ((pkg (dummy-package "xyz-0.1"
+                            (description "xyz is a dummy package."))))
+    (check-description-style pkg)))
+
+(test-equal "description: may start with end of package name"
+  '()
+  (let ((pkg (dummy-package "foobar-xyz"
+                            (description "xyz is a dummy package."))))
+    (check-description-style pkg)))
+
+(test-equal "description: may start with non-hyphenated package name"
+  '()
+  (let ((pkg (dummy-package "foobar-xyz-minimal"
+                            (description "foobar_xyz is a dummy package."))))
+    (check-description-style pkg)))
+
+(test-equal "description: may start with end of package name"
+  '()
+  (let ((pkg (dummy-package "foo-bar"
+                            (description "bar is some thing in foo."))))
+    (check-description-style pkg)))
+
 (test-equal "description: two spaces after end of sentence"
   "sentences in description should be followed by two spaces; possible infraction at 3"
   (single-lint-warning-message
@@ -142,21 +180,21 @@
   '()
   (let ((pkg (dummy-package "x"
                             (description
-                             "E.g. Foo, i.e. Bar resp. Baz (a.k.a. DVD)."))))
+                             "O. Person e.g. Foo, i.e. Bar resp. Baz (a.k.a. DVD).  Name et al. cf. some paper."))))
     (check-description-style pkg)))
 
 (test-equal "description: may not contain trademark signs: ™"
   "description should not contain trademark sign '™' at 20"
   (single-lint-warning-message
    (let ((pkg (dummy-package "x"
-                             (description "Does The Right Thing™"))))
+                             (description "Does The Right Thing™."))))
      (check-description-style pkg))))
 
 (test-equal "description: may not contain trademark signs: ®"
   "description should not contain trademark sign '®' at 17"
   (single-lint-warning-message
    (let ((pkg (dummy-package "x"
-                             (description "Works with Format®"))))
+                             (description "Works with Format®."))))
      (check-description-style pkg))))
 
 (test-equal "description: suggest ornament instead of quotes"
@@ -170,14 +208,14 @@
   "description contains leading whitespace"
   (single-lint-warning-message
    (let ((pkg (dummy-package "x"
-                              (description " Whitespace."))))
+                             (description " Whitespace."))))
      (check-description-style pkg))))
 
 (test-equal "description: trailing whitespace"
   "description contains trailing whitespace"
   (single-lint-warning-message
    (let ((pkg (dummy-package "x"
-                              (description "Whitespace. "))))
+                             (description "Whitespace. "))))
      (check-description-style pkg))))
 
 (test-equal "description: pluralized 'This package'"
@@ -188,12 +226,18 @@
      (check-description-style pkg))))
 
 (test-equal "description: grammar 'allows to'"
-  "description contains typo 'allows to'"
+  "description contains typo 'allows to '"
   (single-lint-warning-message
    (let ((pkg (dummy-package "x"
                              (description "This package allows to do stuff."))))
      (check-description-style pkg))))
 
+(test-equal "description: grammar 'allows to' 2"
+  '()
+  (let ((pkg (dummy-package "x"
+                            (description "This package allows tokenization."))))
+    (check-description-style pkg)))
+
 (test-equal "synopsis: not a string"
   "invalid synopsis: #f"
   (single-lint-warning-message
@@ -276,6 +320,12 @@
       (check-synopsis-style pkg)))
    string<?))
 
+(test-equal "synopsis: starts with texinfo markup"
+  '()
+  (let ((pkg (dummy-package "x"
+                            (synopsis "@code{help}"))))
+    (check-synopsis-style pkg)))
+
 (test-equal "synopsis: too long"
   "synopsis should be less than 80 characters long"
   (single-lint-warning-message
@@ -341,6 +391,80 @@
                              `(#:tests? ,(not (%current-target-system)))))))
     (check-tests-true pkg)))
 
+(test-equal "compiler-for-target: unconditional CC=gcc is unacceptable"
+  "'CC' should be set to '(cc-for-target)' instead of 'gcc'"
+  (single-lint-warning-message
+   (check-compiler-for-target
+    (dummy-package "x" (arguments '(#:make-flags '("CC=gcc")))))))
+
+(test-equal "compiler-for-target: looks through G-expressions"
+  "'CC' should be set to '(cc-for-target)' instead of 'gcc'"
+  (single-lint-warning-message
+   (check-compiler-for-target
+    (dummy-package "x" (arguments
+                        (list #:make-flags #~(list "CC=gcc")))))))
+
+(test-equal "compiler-for-target: (cc-for-target) is acceptable"
+  '()
+  (check-compiler-for-target
+   (dummy-package "x"
+                  (arguments
+                   (list #:make-flags
+                         #~(list (string-append "CC=" (cc-for-target))))))))
+
+(test-equal "compiler-for-target: CC=gcc is acceptable when target=#false"
+  '()
+  (check-compiler-for-target
+   ;; This (dummy) package consists purely of architecture-independent data.
+   (dummy-package "tzdata"
+                  (arguments
+                   (list #:target #false
+                         #:make-flags #~(list "CC=gcc"))))))
+
+(test-equal "misplaced-flags: make-flag is incorrect"
+  "'CMAKE_BUILD_TYPE' should be set with the 'build-type' flag"
+  (single-lint-warning-message
+   (check-misplaced-flags
+     (dummy-package "x"
+                    (arguments
+                     (list #:make-flags #~(list "-DCMAKE_BUILD_TYPE=RELEASE"
+                                                "-DTESTS=ON")))))))
+
+(test-equal "misplaced-flags: configure-flag is incorrect"
+  "'buildtype' should be set with the 'build-type' flag"
+  (single-lint-warning-message
+   (check-misplaced-flags
+     (dummy-package "x"
+                    (arguments
+                     (list #:configure-flags #~(list "--buildtype=release"
+                                                     "--with-tests")))))))
+
+(test-equal "misplaced-flags: cargo feature flags"
+  "'features' should be set with the 'features' flag"
+  (single-lint-warning-message
+   (check-misplaced-flags
+     (dummy-package "x"
+                    (arguments
+                     '(#:cargo-test-flags
+                       '("--features" "force_system_lib")))))))
+
+(test-equal "misplaced-flags: flags without g-exp is incorrect"
+  "'CMAKE_BUILD_TYPE' should be set with the 'build-type' flag"
+  (single-lint-warning-message
+   (check-misplaced-flags
+     (dummy-package "x"
+                    (arguments
+                     '(#:make-flags '("-DCMAKE_BUILD_TYPE=RELEASE"
+                                      "-DTESTS=ON")))))))
+
+(test-equal "misplaced-flags: build-type set correctly"
+  '()
+  (check-misplaced-flags
+    (dummy-package "x"
+                   (arguments
+                    (list #:build-type "RELEASE"
+                          #:make-flags #~(list "-DTESTS=ON"))))))
+
 ;; The emacs-build-system sets #:tests? #f by default.
 (test-equal "tests-true: #:tests? #t acceptable for emacs packages"
   '()
@@ -375,31 +499,29 @@
      (check-inputs-should-be-native pkg))))
 
 (test-equal
-    "inputs: python-setuptools should not be an input at all (input)"
-  "'python-setuptools' should probably not be an input at all"
+    "inputs: python-pip should not be an input at all (input)"
+  "'python-pip' should probably not be an input at all"
   (single-lint-warning-message
    (let ((pkg (dummy-package "x"
-                             (inputs `(("python-setuptools"
-                                        ,python-setuptools))))))
+                             (inputs `(("python-pip" ,python-pip))))))
      (check-inputs-should-not-be-an-input-at-all pkg))))
 
 (test-equal
-    "inputs: python-setuptools should not be an input at all (native-input)"
-  "'python-setuptools' should probably not be an input at all"
+    "inputs: python-pip should not be an input at all (native-input)"
+  "'python-pip' should probably not be an input at all"
   (single-lint-warning-message
    (let ((pkg (dummy-package "x"
                              (native-inputs
-                              `(("python-setuptools"
-                                 ,python-setuptools))))))
+                              `(("python-pip" ,python-pip))))))
      (check-inputs-should-not-be-an-input-at-all pkg))))
 
 (test-equal
-    "inputs: python-setuptools should not be an input at all (propagated-input)"
-  "'python-setuptools' should probably not be an input at all"
+    "inputs: python-pip should not be an input at all (propagated-input)"
+  "'python-pip' should probably not be an input at all"
   (single-lint-warning-message
    (let ((pkg (dummy-package "x"
                              (propagated-inputs
-                              `(("python-setuptools" ,python-setuptools))))))
+                              `(("python-pip" ,python-pip))))))
      (check-inputs-should-not-be-an-input-at-all pkg))))
 
 (test-assert "input labels: no warnings"
@@ -1298,6 +1420,12 @@
   '()
   (check-formatting (dummy-package "x")))
 
+(test-assert "archival: not an origin"
+  (warning-contains? "not an origin"
+                     (check-archival
+                      (dummy-package
+                       "x" (source (computed-file "x-src" #t))))))
+
 (test-assert "archival: missing content"
   (let* ((origin   (origin
                      (method url-fetch)
@@ -1321,7 +1449,8 @@
          ;; https://archive.softwareheritage.org/api/1/content/
          (content  "{ \"checksums\": {}, \"data_url\": \"xyz\",
                       \"length\": 42 }"))
-    (with-http-server `((200 ,content))
+    (with-http-server `((404 "")                  ;extid
+                        (200 ,content))
       (parameterize ((%swh-base-url (%local-url)))
         (check-archival (dummy-package "x" (source origin)))))))
 
@@ -1341,7 +1470,8 @@
                          \"type\": \"file\",
                          \"name\": \"README\"
                          \"length\": 42 } ]"))
-    (with-http-server `((404 "")                  ;lookup-content
+    (with-http-server `((404 "")                  ;lookup-directory-by-nar-hash
+                        (404 "")                  ;lookup-content
                         (200 ,disarchive)         ;Disarchive database lookup
                         (200 ,directory))         ;lookup-directory
       (mock ((guix download) %disarchive-mirrors (list (%local-url)))
@@ -1360,7 +1490,28 @@
                       \"save_request_date\": \"2014-11-17T22:09:38+01:00\",
                       \"save_request_status\": \"accepted\",
                       \"save_task_status\": \"scheduled\" }")
-         (warnings (with-http-server `((404 "No revision.") ;lookup-revision
+         (warnings (with-http-server `((404 "No extid.") ;lookup-directory-by-nar-hash
+                                       (404 "No revision.") ;lookup-revision
+                                       (404 "No origin.")   ;lookup-origin
+                                       (200 ,save))         ;save-origin
+                     (parameterize ((%swh-base-url (%local-url)))
+                       (check-archival (dummy-package "x" (source origin)))))))
+    (warning-contains? "scheduled" warnings)))
+
+(test-assert "archival: missing svn revision"
+  (let* ((origin   (origin
+                     (method svn-fetch)
+                     (uri (svn-reference
+                           (url "http://example.org/svn/foo")
+                           (revision "1234")))
+                     (sha256 (make-bytevector 32))))
+         ;; https://archive.softwareheritage.org/api/1/origin/save/
+         (save     "{ \"origin_url\": \"http://example.org/svn/foo\",
+                      \"save_request_date\": \"2014-11-17T22:09:38+01:00\",
+                      \"save_request_status\": \"accepted\",
+                      \"save_task_status\": \"scheduled\" }")
+         (warnings (with-http-server `((404 "No extid.") ;lookup-directory-by-nar-hash
+                                       (404 "No revision.") ;lookup-revision
                                        (404 "No origin.")   ;lookup-origin
                                        (200 ,save))         ;save-origin
                      (parameterize ((%swh-base-url (%local-url)))
@@ -1378,7 +1529,27 @@
          ;; https://archive.softwareheritage.org/api/1/revision/
          (revision "{ \"author\": {}, \"parents\": [],
                       \"date\": \"2014-11-17T22:09:38+01:00\" }"))
-    (with-http-server `((200 ,revision))
+    (with-http-server `((404 "No directory.")     ;lookup-directory-by-nar-hash
+                        (200 ,revision))
+      (parameterize ((%swh-base-url (%local-url)))
+        (check-archival (dummy-package "x" (source origin)))))))
+
+(test-equal "archival: nar-sha256 extid available"
+  '()
+  (let* ((origin   (origin
+                     (method git-fetch)
+                     (uri (git-reference
+                           (url "http://example.org/foo.git")
+                           (commit "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")))
+                     (sha256 (make-bytevector 32))))
+         ;; https://archive.softwareheritage.org/api/1/extid/doc/
+         (extid   "{ \"extid_type\": \"nar-sha256\",
+                     \"extid\": \"1234\",
+                     \"extid_version\": 0,
+                     \"target\": \"swh:1:dir:cabba93\",
+                     \"target_url\": \"boo\"
+                   }"))
+    (with-http-server `((200 ,extid))
       (parameterize ((%swh-base-url (%local-url)))
         (check-archival (dummy-package "x" (source origin)))))))
 
