@@ -1,8 +1,10 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2012-2022 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2012-2025 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2018 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2021 Maxim Cournoyer <maxim.cournoyer@gmail.com>
+;;; Copyright © 2021 Maxim Cournoyer <maxim@guixotic.coop>
 ;;; Copyright © 2021 Maxime Devos <maximedevos@telenet.be>
+;;; Copyright © 2023 Simon Tournier <zimon.toutoune@gmail.com>
+;;; Copyright © 2025 David Elsing <david.elsing@posteo.net>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -78,6 +80,11 @@
 ;; When grafting, do not add dependency on 'glibc-utf8-locales'.
 (%graft-with-utf8-locale? #f)
 
+(define (bootstrap-binary name)
+  (let ((bin (search-bootstrap-binary name (%current-system))))
+    (and %store
+         (add-to-store %store name #t "sha256" bin))))
+
 
 (test-begin "packages")
 
@@ -285,6 +292,9 @@
          (member (read-at (package-field-location %bootstrap-guile 'version))
                  (let ((version (package-version %bootstrap-guile)))
                    (list version `(version ,version))))
+         (member (read-at (package-field-location coreutils
+                                                  'build-system))
+                 '(build-system gnu-build-system))
          (not (package-field-location %bootstrap-guile 'does-not-exist)))))
 
 ;; Make sure we don't change the file name to an absolute file name.
@@ -367,6 +377,23 @@
           (package-transitive-supported-systems d)
           (package-transitive-supported-systems e))))
 
+(test-equal "package-transitive-supported-systems detects cycles"
+  '("c" "a" "b" "c")
+  (letrec* ((a (dummy-package "a"
+                 (build-system trivial-build-system)
+                 (native-inputs (list c))))
+            (b (dummy-package "b"
+                 (build-system trivial-build-system)
+                 (inputs (list a))))
+            (c (dummy-package "c"
+                 (build-system trivial-build-system)
+                 (inputs (list b)))))
+    (guard (c ((package-cyclic-dependency-error? c)
+               (map package-name
+                    (cons (package-error-package c)
+                          (package-error-dependency-cycle c)))))
+      (package-transitive-supported-systems c))))
+
 (test-assert "package-development-inputs"
   ;; Note: Due to propagated inputs, 'package-development-inputs' returns a
   ;; couple more inputs, such as 'linux-libre-headers'.
@@ -418,12 +445,15 @@
 (let* ((o (dummy-origin))
        (u (dummy-origin))
        (i (dummy-origin))
+       (j (dummy-origin (patches (list o))))
        (a (dummy-package "a"))
        (b (dummy-package "b" (inputs (list a i))))
        (c (package (inherit b) (source o)))
        (d (dummy-package "d"
             (build-system trivial-build-system)
-            (source u) (inputs (list c)))))
+            (source u) (inputs (list c))))
+       (e (dummy-package "e" (source j)))
+       (f (package (inherit e) (inputs (list u)))))
   (test-assert "package-direct-sources, no source"
     (null? (package-direct-sources a)))
   (test-equal "package-direct-sources, #f source"
@@ -437,6 +467,17 @@
       (and (= (length (pk 's-sources s)) 2)
            (member o s)
            (member i s))))
+  (test-assert "package-direct-sources, with patches"
+    (let ((s (package-direct-sources e)))
+      (and (= (length (pk 's-sources s)) 2)
+           (member o s)
+           (member j s))))
+  (test-assert "package-direct-sources, with patches and inputs"
+    (let ((s (package-direct-sources f)))
+      (and (= (length (pk 's-sources s)) 3)
+           (member o s)
+           (member j s)
+           (member u s))))
   (test-assert "package-transitive-sources"
     (let ((s (package-transitive-sources d)))
       (and (= (length (pk 'd-sources s)) 3)
@@ -462,7 +503,7 @@
          (member "boo" keys))))
 
 (test-equal "package-transitive-supported-systems, implicit inputs"
-  %supported-systems
+  (filter target-linux? %supported-systems)
 
   ;; Here GNU-BUILD-SYSTEM adds implicit inputs that build only on
   ;; %SUPPORTED-SYSTEMS.  Thus the others must be ignored.
@@ -470,7 +511,11 @@
                (build-system gnu-build-system)
                (supported-systems
                 `("does-not-exist" "foobar" ,@%supported-systems)))))
-    (parameterize ((%current-system "armhf-linux")) ; a traditionally-bootstrapped architecture
+    ;; For '%current-system', pick an old-style-bootstrap (not full-source
+    ;; bootstrap) architecture, and one that uses a version of
+    ;; 'libstdc++-boot0' that has all of %SUPPORTED-SYSTEMS in its
+    ;; 'supported-systems' field.
+    (parameterize ((%current-system "riscv64-linux"))
       (package-transitive-supported-systems p))))
 
 (test-equal "package-transitive-supported-systems: reduced binary seed, implicit inputs"
@@ -576,14 +621,14 @@
 
 (test-equal "package-source-derivation, origin, sha512"
   "hello"
-  (let* ((bash    (search-bootstrap-binary "bash" (%current-system)))
+  (let* ((bash    (bootstrap-binary "bash"))
          (builder (add-text-to-store %store "my-fixed-builder.sh"
                                      "echo -n hello > $out" '()))
          (method  (lambda* (url hash-algo hash #:optional name
                                 #:rest rest)
                     (and (eq? hash-algo 'sha512)
                          (raw-derivation name bash (list builder)
-                                         #:sources (list builder)
+                                         #:sources (list bash builder)
                                          #:hash hash
                                          #:hash-algo hash-algo))))
          (source  (origin
@@ -602,14 +647,14 @@
 
 (test-equal "package-source-derivation, origin, sha3-512"
   "hello, sha3"
-  (let* ((bash    (search-bootstrap-binary "bash" (%current-system)))
+  (let* ((bash    (bootstrap-binary "bash"))
          (builder (add-text-to-store %store "my-fixed-builder.sh"
                                      "echo -n hello, sha3 > $out" '()))
          (method  (lambda* (url hash-algo hash #:optional name
                                 #:rest rest)
                     (and (eq? hash-algo 'sha3-512)
                          (raw-derivation name bash (list builder)
-                                         #:sources (list builder)
+                                         #:sources (list bash builder)
                                          #:hash hash
                                          #:hash-algo hash-algo))))
          (source  (origin
@@ -626,6 +671,10 @@
     (build-derivations %store (list drv))
     (call-with-input-file output get-string-all)))
 
+(test-equal "package-upstream-name*"
+  (package-upstream-name* (specification->package "guile-gcrypt"))
+  "gcrypt")
+
 
 ;;;
 ;;; Source derivation with snippets.
@@ -1055,10 +1104,33 @@
          (dummy (dummy-package "dummy"
                   (arguments '(#:implicit-inputs? #f))
                   (inputs (list dep*)))))
-    (equal? (package-grafts %store dummy)
-            (list (graft
-                    (origin (package-derivation %store dep))
-                    (replacement (package-derivation %store new)))))))
+    (match (package-grafts %store dummy)
+      ((graft)
+       (and (eq? (graft-origin graft)
+                 (package-derivation %store dep))
+            (eq? (run-with-store %store
+                   (lower-object (graft-replacement graft)))
+                 (package-derivation %store new)))))))
+
+(test-assert "package-grafts, indirect grafts, #:system argument"
+  (let* ((system (if (string=? (%current-system) "riscv64-linux")
+                     "x86_64-linux"
+                     "riscv64-linux"))
+         (new   (dummy-package "dep"
+                  (arguments `(#:implicit-inputs? #f
+                               #:system ,system))))
+         (dep   (package (inherit new) (version "0.0")))
+         (dep*  (package (inherit dep) (replacement new)))
+         (dummy (dummy-package "dummy"
+                  (arguments '(#:implicit-inputs? #f))
+                  (inputs (list dep*)))))
+    (match (package-grafts %store dummy)
+      ((graft)
+       (and (eq? (graft-origin graft)
+                 (package-derivation %store dep system))
+            (eq? (run-with-store %store
+                   (lower-object (graft-replacement graft)))
+                 (package-derivation %store new)))))))
 
 ;; XXX: This test would require building the cross toolchain just to see if it
 ;; needs grafting, which is obviously too expensive, and thus disabled.
@@ -1091,10 +1163,13 @@
          (dummy (dummy-package "dummy"
                   (arguments '(#:implicit-inputs? #f))
                   (inputs (list prop)))))
-    (equal? (package-grafts %store dummy)
-            (list (graft
-                    (origin (package-derivation %store dep))
-                    (replacement (package-derivation %store new)))))))
+    (match (package-grafts %store dummy)
+      ((graft)
+       (and (eq? (graft-origin graft)
+                 (package-derivation %store dep))
+            (eq? (run-with-store %store
+                   (lower-object (graft-replacement graft)))
+                 (package-derivation %store new)))))))
 
 (test-assert "package-grafts, same replacement twice"
   (let* ((new  (dummy-package "dep"
@@ -1113,12 +1188,15 @@
          (p3   (dummy-package "final"
                  (arguments '(#:implicit-inputs? #f))
                  (inputs (list p1 p2)))))
-    (equal? (package-grafts %store p3)
-            (list (graft
-                    (origin (package-derivation %store
-                                                (package (inherit dep)
-                                                         (replacement #f))))
-                    (replacement (package-derivation %store new)))))))
+    (match (package-grafts %store p3)
+      ((graft)
+       (and (eq? (graft-origin graft)
+                 (package-derivation %store
+                                     (package (inherit dep)
+                                              (replacement #f))))
+            (eq? (run-with-store %store
+                   (lower-object (graft-replacement graft)))
+                 (package-derivation %store new)))))))
 
 (test-assert "package-grafts, dependency on several outputs"
   ;; Make sure we get one graft per output; see <https://bugs.gnu.org/41796>.
@@ -1131,17 +1209,24 @@
             (p1  (dummy-package "p1"
                    (arguments '(#:implicit-inputs? #f))
                    (inputs (list p0 `(,p0 "lib"))))))
-    (lset= equal? (pk (package-grafts %store p1))
-           (list (graft
-                   (origin (package-derivation %store p0))
-                   (origin-output "out")
-                   (replacement (package-derivation %store p0*))
-                   (replacement-output "out"))
-                 (graft
-                   (origin (package-derivation %store p0))
-                   (origin-output "lib")
-                   (replacement (package-derivation %store p0*))
-                   (replacement-output "lib"))))))
+    (match (sort (package-grafts %store p1)
+                 (lambda (graft1 graft2)
+                   (string<? (graft-origin-output graft1)
+                             (graft-origin-output graft2))))
+      ((graft1 graft2)
+       (and (eq? (graft-origin graft1) (graft-origin graft2)
+                 (package-derivation %store p0))
+            (eq? (run-with-store %store
+                   (lower-object (graft-replacement graft1)))
+                 (run-with-store %store
+                   (lower-object (graft-replacement graft2)))
+                 (package-derivation %store p0*))
+            (string=? "lib"
+                      (graft-origin-output graft1)
+                      (graft-replacement-output graft1))
+            (string=? "out"
+                      (graft-origin-output graft2)
+                      (graft-replacement-output graft2)))))))
 
 (test-assert "replacement also grafted"
   ;; We build a DAG as below, where dotted arrows represent replacements and
@@ -1208,15 +1293,21 @@
                                (symlink (assoc-ref %build-inputs "p2")
                                         "p2")
                                #t))))))
-    (lset= equal?
-           (package-grafts %store p3)
-           (list (graft
-                   (origin (package-derivation %store p1 #:graft? #f))
-                   (replacement (package-derivation %store p1r)))
-                 (graft
-                   (origin (package-derivation %store p2 #:graft? #f))
-                   (replacement
-                    (package-derivation %store p2r #:graft? #t)))))))
+    (match (package-grafts %store p3)
+      ((graft1 graft2)
+       (and (eq? (graft-origin graft1)
+                 (package-derivation %store p1 #:graft? #f))
+            (eq? (run-with-store %store
+                   (lower-object (graft-replacement graft1)))
+                 (package-derivation %store p1r #:graft? #t))
+            (eq? (graft-origin graft2)
+                 (package-derivation %store p2 #:graft? #f))
+            ;; XXX: Remove parameterize when
+            ;; <https://issues.guix.gnu.org/75879> is fixed.
+            (eq? (parameterize ((%graft? #t))
+                   (run-with-store %store
+                     (lower-object (graft-replacement graft2))))
+                 (package-derivation %store p2r #:graft? #t)))))))
 
 ;;; XXX: Nowadays 'graft-derivation' needs to build derivations beforehand to
 ;;; find out about their run-time dependencies, so this test is no longer
@@ -1241,8 +1332,9 @@
 ;;                               #:guile guile))))
 
 (test-equal "package->bag"
-  `("foo86-hurd" #f (,(package-source gnu-make))
-    (,(canonical-package glibc)) (,(canonical-package coreutils)))
+  (parameterize ((%current-system "foo86-hurd"))
+    `("foo86-hurd" #f (,(package-source gnu-make))
+      (,(canonical-package glibc)) (,(canonical-package coreutils))))
   (let ((bag (package->bag gnu-make "foo86-hurd")))
     (list (bag-system bag) (bag-target bag)
           (assoc-ref (bag-build-inputs bag) "source")
@@ -1464,6 +1556,36 @@
       ((("python" python) _ ...)
        (derivation-file-name (package-derivation %store python))))))
 
+(test-assert "package-input-rewriting, recursive"
+  (let* ((dep (dummy-package "dep" (native-inputs (list grep))))
+         (p0  (dummy-package "example1" (inputs (list dep grep))))
+         (p1  (dummy-package "example2" (inputs (list dep grep))))
+         (replacements `((,grep . ,findutils) (,p0 . ,p1)))
+         (rewrite (package-input-rewriting replacements))
+         (rewrite/recursive (package-input-rewriting replacements
+                                                     #:recursive? #t))
+         (p2 (rewrite p0))
+         (p3 (rewrite/recursive p0)))
+    (and (string=? (package-name p2) "example2")
+         ;; Here P0 is replaced by P1, but P1 itself is kept unchanged.
+         (match (package-inputs p2)
+           ((("dep" dep1) ("grep" dep2))
+            (and (match (package-native-inputs dep1)
+                   ((("grep" x)) (eq? x grep)))
+                 (eq? dep2 grep))))
+
+         ;; Here P0 is replaced by P1, and in addition references to GREP in
+         ;; P1 and its dependencies are also replaced by FINDUTILS.
+         (string=? (package-name p3) "example2")
+         (match (package-inputs p3)
+           ((("dep" dep1) ("grep" dep2))
+            (and (match (package-native-inputs dep1)
+                   ((("grep" x))
+                    (string=? (package-full-name x)
+                              (package-full-name findutils))))
+                 (string=? (package-full-name dep2)
+                           (package-full-name findutils))))))))
+
 (test-assert "package-input-rewriting/spec"
   (let* ((dep     (dummy-package "chbouib"
                     (native-inputs `(("x" ,grep)))))
@@ -1573,6 +1695,39 @@
     (match (delete-duplicates pythons eq?)
       ((p) (eq? p (rewrite python))))))
 
+(test-assert "package-input-rewriting/spec, hidden package"
+  ;; Hidden packages are not subject to rewriting.
+  (let* ((python  (hidden-package python))
+         (p0      (dummy-package "chbouib"
+                    (build-system trivial-build-system)
+                    (inputs (list python))))
+         (rewrite (package-input-rewriting/spec
+                   `(("python" . ,(const sed)))
+                   #:deep? #t))
+         (p1      (rewrite p0))
+         (bag1    (package->bag p1))
+         (pythons (filter-map (match-lambda
+                                (("python" python) python)
+                                (_ #f))
+                              (bag-transitive-inputs bag1))))
+    (match (delete-duplicates pythons eq?)
+      ((p) (eq? p python)))))
+
+(test-assert "package-input-rewriting/spec, replace hidden package"
+  ;; Rewrite hidden packages when requested.
+  (let* ((python  (hidden-package python))
+         (p0      (dummy-package "chbouib"
+                    (build-system trivial-build-system)
+                    (inputs (list python))))
+         (rewrite (package-input-rewriting/spec
+                   `(("python" . ,(const sed)))
+                   #:replace-hidden? #t))
+         (p1      (rewrite p0)))
+    (match (package-inputs p1)
+      ((("python" python))
+       (and (string=? (package-full-name python)
+                      (package-full-name sed)))))))
+
 (test-equal "package-input-rewriting/spec, graft"
   (derivation-file-name (package-derivation %store sed))
 
@@ -1737,6 +1892,9 @@
     ;; exponential behavior.
     (let ((set1 duplicates1 (list->set* from-cache))
           (set2 duplicates2 (list->set* no-cache)))
+      ;; For easier debugging.
+      (pk 'from-cache-duplicates: duplicates1)
+      (pk 'no-cache-duplicates: duplicates2)
       (and (null? duplicates1) (null? duplicates2)
            (every (cut set-contains? set1 <>) no-cache)
            (every (cut set-contains? set2 <>) from-cache)))))
@@ -1978,6 +2136,14 @@
    (dummy-package "a"
      (arguments (this-package-native-input "hello")))))
 
+(test-equal "this-package-input, origin"
+  "http://example.org/foo.tar.gz"
+  (origin-uri
+   (package-arguments
+    (dummy-package "a"
+      (inputs (list (dummy-origin (uri "http://example.org/foo.tar.gz"))))
+      (arguments (this-package-input "foo.tar.gz"))))))
+
 (test-eq "modify-inputs, replace"
   coreutils
   ;; Replace an input; notice that the label in unchanged.
