@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2013-2022 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2013-2023 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2014 Alex Kost <alezost@gmail.com>
 ;;;
 ;;; This file is part of GNU Guix.
@@ -382,6 +382,28 @@
        (_          (built-derivations (list drv))))
     (return (file-exists? (string-append bindir "/guile")))))
 
+(test-assertm "profile-derivation, #:system, and hooks"
+  ;; Make sure all the profile hooks are built for the system specified with
+  ;; #:system, even if that does not match (%current-system).
+  ;; See <https://issues.guix.gnu.org/65225>.
+  (mlet* %store-monad
+      ((system -> (if (string=? (%current-system) "riscv64-linux")
+                      "x86_64-linux"
+                      "riscv64-linux"))
+       (entry -> (package->manifest-entry packages:coreutils))
+       (_        (set-guile-for-build (default-guile) system))
+       (drv      (profile-derivation (manifest (list entry))
+                                     #:system system))
+       (refs     (references* (derivation-file-name drv))))
+    (return (and (string=? (derivation-system drv) system)
+                 (pair? refs)
+                 (every (lambda (ref)
+                          (or (not (string-suffix? ".drv" ref))
+                              (let ((drv (read-derivation-from-file ref)))
+                                (string=? (derivation-system drv)
+                                          system))))
+                        refs)))))
+
 (test-assertm "profile-derivation relative symlinks, one entry"
   (mlet* %store-monad
       ((entry ->   (package->manifest-entry %bootstrap-guile))
@@ -441,7 +463,6 @@
        (target ->   "arm-linux-gnueabihf")
        (grep        (package->cross-derivation packages:grep target))
        (sed         (package->cross-derivation packages:sed target))
-       (locales     (package->derivation packages:glibc-utf8-locales))
        (drv         (profile-derivation manifest
                                         #:hooks '()
                                         #:locales? #t
@@ -453,15 +474,11 @@
                  (and (string-suffix? name input) input)))
              (derivation-inputs drv))))
 
-    ;; The inputs for grep and sed should be cross-build derivations, but that
-    ;; for the glibc-utf8-locales should be a native build.
     (return (and (string=? (derivation-system drv) (%current-system))
                  (string=? (find-input packages:grep)
                            (derivation-file-name grep))
                  (string=? (find-input packages:sed)
-                           (derivation-file-name sed))
-                 (string=? (find-input packages:glibc-utf8-locales)
-                           (derivation-file-name locales))))))
+                           (derivation-file-name sed))))))
 
 (test-assert "package->manifest-entry defaults to \"out\""
   (let ((outputs (package-outputs packages:glibc)))
@@ -504,6 +521,17 @@
            (manifest-entry-search-paths
             (package->manifest-entry mpl)))))
 
+(test-assert "package->manifest-entry, origin in propagated inputs"
+  ;; See <https://codeberg.org/guix/guix/pulls/969>
+  (let ((p1 (dummy-package "pkg"))
+        (p2 (dummy-package "pkg"
+              (propagated-inputs
+               (list (package-source %bootstrap-guile)))))
+        (package->manifest-entry-dependencies
+         (compose manifest-entry-dependencies package->manifest-entry)))
+    (and (null? (package->manifest-entry-dependencies p1))
+         (null? (package->manifest-entry-dependencies p2)))))
+
 (test-assert "packages->manifest, no duplicates"
   (let ((expected
          (manifest
