@@ -1,7 +1,7 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2015 David Thompson <davet@gnu.org>
 ;;; Copyright © 2020 by Amar M. Singh <nly@disroot.org>
-;;; Copyright © 2016-2022 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2016-2022, 2024 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -25,7 +25,8 @@
   #:use-module (guix scripts publish)
   #:use-module (guix tests)
   #:use-module (guix config)
-  #:use-module (guix utils)
+  #:use-module ((guix utils) #:select (call-with-temporary-directory))
+  #:use-module ((guix build utils) #:select (call-with-temporary-output-file))
   #:use-module (gcrypt hash)
   #:use-module (guix store)
   #:use-module (guix derivations)
@@ -95,6 +96,20 @@
         (lambda ()
           exp ...)))))
 
+;; guix-publish uses (current-processor-count) as the default number of
+;; workers, however on a system with a large number of cores, that large
+;; number of worker threads being used in the course of these tests can end up
+;; hitting resource limits and causing spurious test failures.
+;;
+;; This will depend on what resource limits are in use, but 64 seems low
+;; enough to be able to run the tests without problems.
+(let ((max-processors 64))
+  (when (> (current-processor-count)
+           max-processors)
+    (setaffinity
+     (getpid)
+     (make-bitvector max-processors #t))))
+
 ;; Run a local publishing server in a separate thread.
 (with-separate-output-ports
  (call-with-new-thread
